package pinball;

import pinball.*;
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.buffer.*;
import pinball.menuevent.*;

import pinball.player.standard.*;

import java.awt.*;
import java.io.*;

/**
 * Starts up the pinball program with the type of player that logs in.
 */

public class JavaQcam {
	/** The main menu of the program. Used to start,exit the program. */			
	protected MainMenu menu=null;
	
	/** Used to check if the files are found. */
	private boolean fileFound=false;

	/** Generate the camera. Used to watch the pinball game. */
	protected Camera Qcam;

	/** Game options. */
	private Options options=null;
		
	/* Output file, for game options. */	

	/** Output file, for game options. */
	private FileOutputStream out=null;
	
	/** Output file, for game options. */
        private DataOutputStream logout=null;

	/* Input file, for game options. */
 	
	/** Input file, for game options. */
	private FileInputStream in=null;
	
	/** Input file, for game options. */
        private DataInputStream login=null;
       
        /** Name and location of the options file. */
       	private String optionsFile="/u/goetz/checkout/pinball/options.dat";
        
        /** The log file. */
       	private PrintStream statsLog=null;
        
        /** Is the log file to be used. */
        private boolean logfile=false;
        
        /** The player to play pinball. */
        protected StdPlayer player=null;
	
	/** Name of the flipper file that contains the locations of the flippers. */
	private String flippersFile=null;

	/** Name of the batch mode file that contains the configuration. */
	private String batchFile=null;
	
	/* Input file, for flippers. */
	
	/** Input file, for flippers. */
 	private FileInputStream flip=null;
	
	/** Input file, for flippers. */
        private DataInputStream flipin=null;
	
	/** Input file, for batch mode. */
 	private FileInputStream batch=null;
	
	/** Input file, for batch mode. */
        private DataInputStream batchin=null;

	/** How the player is going to play. */
	protected String how;

	/** For extended classes. */
	public JavaQcam(){}
	
	/**
	 * Starts the pinball program.
	 *
	 * @param args The command line arguments given.
	 * @param player The player of the game.
	 * @param how How the player will player. Local, Client, or Server.
	 */
	public JavaQcam(String[] args, StdPlayer player, String how) {
		System.err.println("Starting Pinball System.");
		this.player=player;
		Qcam=player.getCamera();
		GameOptions.setArgs(args);				
						
		begin(args,how);
	}
	
	/** 
	 * Start up the program. Takes command line arguments.
	 * @param args The command line arguments given.
	 * @param how How the player will player. Local, Client, or Server.
	*/
	protected void begin(String[] args, String how) {
					
		this.how=how;
		
		if(GameOptions.play())
			GameOptions.dataPort("/dev/null");
		
	        /* Check command line arguments. */
	        for(int i=0;i<args.length;i++) {
			char []command; // Command line argument.
			char []value;	// Value of arguement.
			
			command=args[i].toCharArray();
			value=new char[command.length-2];
						
			/* Get value of command. */
			for(int j=2;j<command.length;j++)
				value[j-2]=command[j];
			/* Check command. */
			if(command[0]=='-') {
				/* Server. */
				if(command[1]=='S') {
					System.err.println("Server mode.");
					/* Set the options file to the server's file. */
					optionsFile="/u/goetz/checkout/pinball/soptions.dat";
				}
				
				/* Log file, no video. */
				if(command[1]=='l') {
					logfile=true;
					PinballLog.setPath(new String(value),false);				
				}
				
				/* Log file, with video. */
				if(command[1]=='L') {
					logfile=true;
					PinballLog.setPath(new String(value),true);				
 				}


				/* Flipper file. */
				if(command[1]=='F') {
					flippersFile=new String(value);
				}

				/* Flipper file. */
				if(command[1]=='B') {
					batchFile=new String(value);
					GameOptions.batchMode(true);
					GameOptions.post(false);
				}
				
			}	
	
		}
	        
	      	/* If the system is not in batch mode, display the options file. */
		if(!GameOptions.batchMode()) {
			/* Open the game options. */
			try {in = new FileInputStream(optionsFile);}
			catch (Exception e){fileFound=false;}
		
			/* Check if the file existed.*/
			if(in!=null)
				fileFound=true;
			
			/* Generate the default game options. */
		        options=new Options(this);       					
	
			/* If the game file is found, read from it. This sets up the options
			previously saved. */
			if(fileFound) {
				try { login = new DataInputStream(in);}
				catch (Exception e){System.err.println("4: "+e);}
				options.read(login);
			}
		}

		/* Setup flippers file is called for. */
		if(flippersFile!=null) {
			fileFound=false;
	
			/* Open the flippers file. */
			try {flip = new FileInputStream(flippersFile);}
			catch (Exception e){fileFound=false;}
		
			/* Set the name of the flipper file to write to.*/
			GameOptions.writeFlippersFile(flippersFile);
			GameOptions.readFlippersFile(flippersFile);
			
			
			try{flip.close();}
			catch (Exception e){}
			
	
		}
		
		if(GameOptions.batchMode()) {
			fileFound=false;
			
			/* Open the batch mode. */
			try {batch = new FileInputStream(batchFile);}
			catch (Exception e){fileFound=false;System.err.println(e);}
		
			/* Check if the file existed.*/
			if(batch!=null)
				fileFound=true;
				
			/* If the game file is found, read from it. This sets up the options
			previously saved. */
			if(fileFound) {
				try { batchin = new DataInputStream(batch);}
				catch (Exception e){System.err.println("4: "+e);}
			}
			
			else {
				System.err.println("Batch file "+batchFile+" is not found.");
				System.exit(-1);
			}
			
			new BatchMode(batchin);
			next();			
		}
		
		/* Waiting for the system to start. */						
		System.err.println("Waiting...");
	}					

	
	/** Used by the Options menu to continue with the program. */	
	protected void next() {

		/* Save the game options. */
		if(!GameOptions.batchMode()){
			try {out = new FileOutputStream(optionsFile);}
			catch (Exception e){System.err.println("1: "+e);}
			try { logout = new DataOutputStream(out);}
			catch (Exception e){System.err.println("2: "+e);}
			options.save(logout);
	
			/* Take away the game options window. */
			options.toBack();
			options.dispose();
		}
					
		/* If recording the log file, create it. */
		if(logfile) {
			/* Set the id of the log file. */
			PinballLog.setStatsFile(player.id(),1);
			System.err.println("Logging Game");
		}

		/* Write the player id to the log file. */
		PinballLog.headerInfo("PLAYER="+player.id(),this);
	
		Qcam.cameraInfo();
		
		createMenu();		
		
		/* Start the program. */
		menu.begin();
		
		System.err.println("Program running.");
	}	

	public void createMenu() {
		/* Make the display. */
		menu=new MainMenu(Qcam,player,how);
	}
}	
		
