package pinball;

import pinball.*;
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.buffer.*;
import pinball.menuevent.*;

import pinball.player.standard.*;

import java.awt.*;
import java.io.*;
import java.security.*;
import java.net.*;

/**
 * Used for the web submissions to restrict what people can do on the system. 
 */
public class PinballSecurityManager extends SecurityManager {
	/** Where the output will be stored. */
	private String writeDirectory;
	
	/** The location of the object files that are read in. */
	private String readDirectory="/u/handin/pinball/input/java/";
	
	/** 
	 * Start the security manager with the location of the output directory.
	 *
	 * @param writeDirectory where to write the output to.	
	*/
    	public PinballSecurityManager(String writeDirectory) {
    		this.writeDirectory=writeDirectory;
		File check=new File(writeDirectory);
		String path="";
		
		/* Get the real path name for the file. */
		try{
			path=check.getCanonicalPath();
		}
		catch(Exception e){}
    		this.writeDirectory=path;

		check=new File(readDirectory);
		path="";
		
		/* Get the real path name for the file. */
		try{
			path=check.getCanonicalPath();
		}
		catch(Exception e){}
    		this.readDirectory=path;
    	}
	
	/**
	 * For extended classes.
	 */
	public PinballSecurityManager() {
    		this.writeDirectory="";
    	}

    	public void checkCreateClassLoader() {} 
	public void checkAccess(Thread g) {}
    	public void checkAccess(ThreadGroup g) { }
    	public void checkExit(int status) { }

	/** No executing of commands. */	
    	public void checkExec(String cmd) {throw new SecurityException("Can't execute anything!");}

	/** Allow certain library files to load. */
    	public void checkLink(String lib) {checkMyLink(lib);}

	/** No file descriptors. */
    	public void checkRead(FileDescriptor fd) {throw new SecurityException("No FileDescriptors allowed");}
	
	/** Only certain classes can read from certain directories. */
    	public void checkRead(String file) {checkMyRead(file);}
	
	/** Only certain classes can read from certain directories. */
    	public void checkRead(String file, Object context) {checkMyRead(file);}
    	
	/** Only certain classes can write to certain directories. */
	public void checkWrite(FileDescriptor fd) {throw new SecurityException("No FileDescriptors allowed");}

	/** Only certain classes can write to certain directories. */
    	public void checkWrite(String file) {checkMyWrite(file);}
	
	/** Deleting of files is not allowed. */	
    	public void checkDelete(String file) {throw new SecurityException("You can't delete anything!");}

	/** No network connections are allowed. */
    	public void checkConnect(String host, int port) {throw new SecurityException("You can't make connections!");}

	/** No network connections are allowed. */
    	public void checkConnect(String host, int port, Object context) {throw new SecurityException("You can't make connections!");}

	/** No network connections are allowed. */
    	public void checkListen(int port) {throw new SecurityException("You can't make connections!");}

	/** No network connections are allowed. */
    	public void checkAccept(String host, int port) {throw new SecurityException("You can't make connections!");}

	/** No network connections are allowed. */
    	public void checkMulticast(InetAddress maddr) {throw new SecurityException("You can't make connections!");}

	/** No network connections are allowed. */ 
    	public void checkMulticast(InetAddress maddr, byte ttl) {throw new SecurityException("You can't make connections!");}

    	public void checkPropertiesAccess() {}
    	public void checkPropertyAccess(String key) {}
    	public void checkPropertyAccess(String key, String def) {}
    	public boolean checkTopLevelWindow(Object window) { return true; }

	/** No printing is allowed. */
    	public void checkPrintJobAccess() {throw new SecurityException("Can't print!");}

	/** Cannot use the clipboard. */
    	public void checkSystemClipboardAccess() {throw new SecurityException("Can't access clipboard!");}

	/** No access to the event queue. */
    	public void checkAwtEventQueueAccess() {throw new SecurityException("Can't access the event queue!");}

    	public void checkPackageAccess(String pkg) { }
    	public void checkPackageDefinition(String pkg) { }
    	public void checkSetFactory() { }
    	public void checkMemberAccess(Class clazz, int which) { }
    	public void checkSecurityAccess(String provider) { }

    
	/** 
	 * Check to see if the file to be written to is in the allowed directory.
	 *
	 * @param file the file to be checked.
	 */
    	private void checkMyWrite(String file) {
		File check=new File(file);
		String path="";
		
		/* Get the real path name for the file. */
		try{
			path=check.getCanonicalPath();
			path=new File(path).getParent();
		}
		catch(Exception e){}
		
		/* Only these classes can write. */
		if(!inClass("pinball.part.BallReturn") &&
			!inClass("pinball.part.Camera") && 
			!inClass("pinball.part.Coin") &&
			!inClass("pinball.part.Flipper") &&
			!inClass("pinball.part.Plunger") &&
			!inClass("pinball.part.StartButton") &&
			!inClass("pinball.io.SerialPort") &&
			!inClass("pinball.PinballLog")) {
				throw new SecurityException("This Class Cannot write to: "+path);
			}
		
		/* Only these directories or the serial port can be accesed. */
		if(path.compareTo(writeDirectory)!=0 && file.compareTo("/dev/cuaa0")!=0)
			throw new SecurityException("Cannot write to: "+path+"\nYou can only write to "+writeDirectory);
	}
	
	/** 
	 * Check to see if the file to be read from is in the allowed directory.
	 *
	 * @param file the file to be checked.
	 */
	private void checkMyRead(String file) {
    		File check=new File(file);
		String path="";
		
		/* Get the real file path. */
		try{
			path=check.getCanonicalPath();
			path=new File(path).getParent();
		}
		catch(Exception e){}
		
		/* Only the serial port, the standard sercurity file, or the directory allowed can be accessed. */
		if(file.compareTo("/dev/cuaa0")!=0 && path.compareTo("/usr/local/jdk1.1.8/lib/security")!=0 && path.compareTo(readDirectory)!=0) {
			throw new SecurityException("Cannot read from: "+path);
		}
		
		/* Only these classes can read files. */
		if(!inClass("pinball.part.BallReturn") &&
			!inClass("pinball.part.Camera") && 
			!inClass("pinball.part.Coin") &&
			!inClass("pinball.part.Flipper") &&
			!inClass("pinball.part.Plunger") &&
			!inClass("pinball.part.StartButton") &&
			!inClass("pinball.io.SerialPort") &&
			!inClass("pinball.JavaQcam")) {
				throw new SecurityException("Cannot read from: "+path);
			}
	}

	/** Only certain library files can be opened. 
	 * 
	 * @param lib the library file to be checked for.
	 */
	private void checkMyLink(String lib) {
		/* Only the camera and awt libaries are allowed. */
		if(lib.compareTo("Camera")!=0 && lib.compareTo("awt")!=0)
			throw new SecurityException("Cannot use library: "+lib);
	}	
}	
