package pinball.applet.human;

import pinball.*;
import pinball.buffer.*; 
import pinball.player.*;
import pinball.player.remote.*;
import pinball.server.*;

import java.awt.*;
import java.applet.Applet;


/**
 * Human player plays pinball. 
 */
public class HumanStrategy extends Strategy {	

	/** The command from the human to flip the flippers. */
	protected FlipperCommands commands;
	
	/** The previous flip status of the flippers. */
	protected boolean lastLeft=false, lastRight=false;

	/** Frame frame for requesting images. */
	protected int frameRate=100;
	
	/** Frame number. */
	protected int frame=0;
	
	/** The area to draw to. */
	protected HumanCanvas humanCanvas;	

	/** 
	 * The strategy thread for the player. 
	 *
 	 * @param a applet
	 */		
	public HumanStrategy(Applet a,MyClient client) {
		this.stub=new RemoteStub(this,client);
		id="human";				
		Dimension d=a.getSize();   		
		humanCanvas=new HumanCanvas(d.width,d.height);
		//new RemoteExit((RemoteStub)stub);
		
		a.add(humanCanvas, BorderLayout.CENTER);

		humanCanvas.setup();

		commands=humanCanvas.getFlipperCommands();
		
		humanCanvas.requestFocus();
	}	
 				
	/** For extended classes. */			
	public HumanStrategy() {

	}
	
	
	/**  
	 * Don't let the flippers reset on their own. The human player will reset them.
	 */ 
	public void flippersReset() {}
		
	/** Return the image from the stub. */
	public Image getImage() {
		return stub.getImage();
	}		
	
	/** Have the human presss the keys to raise of lower the flippers. */		
	public void decision() {
		/* Get the ball's location in the frame. */
		x=buffinfo.blip().x();
		y=buffinfo.blip().y();
	
		/* Add in the image offset to find the location	of the ball
		   in the area of the pinball machine.	*/
		if(x!=-1)	
			x+=buffinfo.XOffset();
		if(y!=-1)	
			y+=buffinfo.YOffset();
	
		/* Only request images at the frame rate. */		
		if(frameRate!=0 && frame%frameRate==0) {
			stub.requestFrame();
		}
		
		Image image=stub.getImage();	
		if(image!=null)
			humanCanvas.setImage(image);

		humanCanvas.setBall(x,y);
		
			
		if(lastLeft!=commands.left()) {
			if(commands.left())
				stub.leftFlipperState(true);
			if(!commands.left())
				stub.leftFlipperState(false);
		}
		lastLeft=commands.left();

		if(lastRight!=commands.right()) {
			if(commands.right())
				stub.rightFlipperState(true);		
			if(!commands.right())
				stub.rightFlipperState(false);
		}
		lastRight=commands.right();
		
		humanCanvas.doDraw();
			
		frame++;	
	}
	

}
