package pinball.buffer;

import java.io.*;
import java.util.*;

/** This is a frame grabed from the camera. It also has the dimensions of the frame.
*/
public class CameraFrame implements Serializable {
	/** The frame from the camera. */
	protected  byte [] frame;
	
	/** The id of the frame. */	
	protected int id;
	
	/** The width of the frame. */
	protected int width;
	
	/** The height of the frame. */
	protected int height;
	
	/** The xoffset of the frame. */
	protected int xoffset;
	
	/** The yoffset of the frame. */
	protected int yoffset;
	
	/** The transfer size for the frame. */
	protected int transfer;
	
	/** 
	 * Create a frame to be grabed by the camera.
	 * 
	 * @param id the frame's id
	 * @param size the number of bytes of the frame.
	 * @param width the width of the frame.
	 * @param height the height of the frame.
	 * @param xoffset the xoffset of the frame.
	 * @param yoffset the yoffset of the frame.
 	 * @param transfer the transfer size of the frame.
	 */
	public CameraFrame(int id, int size, int width, int height, int xoffset, int yoffset, int transfer) {
		this.id=id;
		frame=new byte[size];
		this.width=width;
		this.height=height;
		this.xoffset=xoffset;
		this.yoffset=yoffset;	
		this.transfer=transfer;	
	}
	
	/** 
	 * Create a frame that was grabed by the camera.
	 * 
	 * @param id the frame's id
	 * @param frame the bytes of the frame.
	 * @param width the width of the frame.
	 * @param height the height of the frame.
	 * @param xoffset the xoffset of the frame.
	 * @param yoffset the yoffset of the frame.
	 * @param transfer the transfer size of the frame.	 
	 */
	public CameraFrame(int id, byte frame[], int width, int height, int xoffset, int yoffset, int transfer) {
		this.id=id;
		this.frame=frame;
		this.width=width;
		this.height=height;
		this.xoffset=xoffset;
		this.yoffset=yoffset;	
		this.transfer=transfer;
	}	
	
	/** 
	 * Create a frame that was grabed by the camera.
	 * 
	 * @param id the frame's id
	 * @param frame the bytes of the frame.
	 * @param width the width of the frame.
	 * @param height the height of the frame.
	 * @param xoffset the xoffset of the frame.
	 * @param yoffset the yoffset of the frame.
	 */
	public CameraFrame(int id, byte frame[], int width, int height, int xoffset, int yoffset) {
		this.id=id;
		this.frame=frame;
		this.width=width;
		this.height=height;
		this.xoffset=xoffset;
		this.yoffset=yoffset;	
		this.transfer=1;
	}	

	/**
	 * Get the id of the frame. 
	 *
	 * @return the id of the frame.
	 */
	public int getID() {
		return id;
	}
	
	/** Set the ID. 
	* @param id the id of the frame. 
	*/
	public void setID(int id) {
		this.id=id;
	}
	
	/**
	 * Get the frame. 
	 *
	 * @return frame to store image.
	 */
	public byte [] getFrame() {
		return  frame;
	}
	
	/**
	 * Get the width of the frame. 
	 *
	 * @return width of the frame.
	 */	
	public int getWidth() {
		return width;
	}

	/**
	 * Get the height of the frame. 
	 *
	 * @return height of the frame.
	 */		
	public int getHeight() {
		return height;
	}
	
	/**
	 * Get the xoffset of the frame. 
	 *
	 * @return xoffset of the frame.
	 */	
	public int getXOffset() {
		return xoffset;
	}
	
	/**
	 * Get the yoffset of the frame. 
	 *
	 * @return yoffset of the frame.
	 */	
	public int getYOffset() {
		return yoffset;
	}		
	
	/**
	 * Get the transfer size of the frame. 
	 *
	 * @return transfer size of the frame.
	 */	
	public int getTransfer() {
		return transfer;
	}		
}
