package pinball.log;

import pinball.*;
import pinball.buffer.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;

/**
 * A frame read from the camera file. 
 */
public class SavedFrame {
	/** Frame from the camera. */
	private CameraFrame frame;
	
	/** Text from the frame. */	
	private String text;
	
	/** Location of the ball. */	
	private int x, y;
	
	/** How long to wait between frames. */	
	private long sleepTime;

	/** 
	 * Store a frame from the video log file into the memory.
	 *
	 * @param frame Frame from the camera.
	 * @param text 	Text from the frame.
	 * @param x 	x location of the ball.
	 * @param y	y location of the ball.
	 * @param sleepTime	how long to wait between frames.
	 */
	public SavedFrame(CameraFrame frame, String text, int x, int y, long sleepTime) {
		this.frame=frame;
		this.text=text;
		this.x=x;
		this.y=y;
		this.sleepTime=sleepTime;
	}
	
	/**
	 * Get the frame.
	 *
	 * @return camera frame.
	 */
	public CameraFrame frame() {
		return frame;
	}
	
	/**
	 * Get the text for the frame.
	 *
	 * @return text from the frame.
	 */
	public String text() {
		return text;
	}
	
	/**
 	 * Get the x location of the ball.
	 *
	 * @return the x location of the ball.
	 */
	public int x() {
		return x;
	}

	/**
 	 * Get the y location of the ball.
	 *
	 * @return the y location of the ball.
	 */	public int y() {
		return y;
	}
	
	public long sleepTime() {
		return sleepTime;
	}
}
