package pinball.menuevent;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.player.*;

import java.awt.event.*;
import java.awt.*;

/**
 * Start up the pinball game.
 */
public class PinballStart implements ActionListener {
	/* The threads in the system. */
	
	/** Post thread. */
	private	Post post;
	
	/** Framegrabber thread. */
	private	FrameGrabber frameGrabber;
	
	/** Analyze thread. */
	private Analyze analyze;
	
	/** Strategy thread. */
	private Strategy strategy;
		
	/* Buttons in the window. */
		
	/** The exit system button. */
	private Button exit;
	
	/** The start system button. */
	private Button start;
	
	/** The video verbose button. */	
	private Button verbose;
	
	
	/** The main menu. */
	private MainMenu mm;
	
	/**
	 * Set up the event for what happens when the game is started.
	 *
	 * @param mm window of the program
 	 * @param drawing draawing thread
	 * @param analyzeImage analyze thread
	 * @param playStrategy strategy
	 * @param start start button
	 */	 
	public PinballStart(MainMenu mm, Post post, FrameGrabber frameGrabber,
	Analyze analyze, Strategy strategy, Button start) { 
		this.post=post;
		this.frameGrabber=frameGrabber;
		this.analyze=analyze;
		this.strategy=strategy;
		this.start=start;
		this.mm=mm;
	}
	
	/**
	 * The game is started.
	 *
	 *
	 */
    	public void actionPerformed(ActionEvent e) {
    		start();
    	}
	
	/**
	 * Start the game. 
	 */
	public void start() {
		/* Don't start the game again, if it's already running. */
    		if(GameOptions.running()) {
    			return;
    		}
    		GameOptions.running(true);
    		
		/* Game Options. Write to the statistics log.*/
		PinballLog.headerInfo("postThread="+GameOptions.post(),this);
		PinballLog.headerInfo("strategyThread="+GameOptions.strategy(),this);
		PinballLog.headerInfo("analyzeThread="+GameOptions.analyze(),this);
		PinballLog.headerInfo("recording="+GameOptions.record(),this);
		PinballLog.headerInfo("playing="+GameOptions.play(),this);
		PinballLog.headerInfo("pixelRate="+GameOptions.pixelRate(),this);
		PinballLog.headerInfo("frameRate="+GameOptions.frameRate(),this);
	 	PinballLog.headerInfo("bufferSize="+GameOptions.bufferSize(),this);
		PinballLog.headerInfo("flipperResetTime="+GameOptions.resetTime(),this);
		PinballLog.headerInfo("ballMin="+GameOptions.ballMin(),this);
		PinballLog.headerInfo("ballMax="+GameOptions.ballMax(),this);
		PinballLog.headerInfo("findFlippers="+GameOptions.findFlippers(),this);
		PinballLog.headerInfo("moveThreshold="+GameOptions.moveThreshold(),this);
			
			
		/* Start up the threads that are defined in GameOptions. */
		frameGrabber.start();		
		if(GameOptions.analyze())
			analyze.start();
		if(GameOptions.strategy())
			strategy.start();
		if(GameOptions.post())
			post.start();
			
		start.setForeground(Color.gray);
	}
}
