/*

qcread.c Connectix QuickCam hardware interface

Copyright (C) 1996, 1997 Alex Belits

This software contains ideas and modified parts from qcam,
written by Scott Laird.

Copyright (C) 1996 by Scott Laird

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL SCOTT LAIRD OR ALEX BELITS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include "getopt.h"
#include "cqc.h"

#define VERSION "0.2"
#define RELEASEDATE "May 19 1997"

struct qcread_qcam *qc;

char *versionline=
"qcread version " VERSION "\n"
"released " RELEASEDATE ", compiled "__DATE__" " __TIME__"\n";

char *helpmessage=
"Connectix QuickCam image reader, available options:\n"
"-V, --version,         print version number\n"
"-f, --config FILE      config file name FILE\n"
"                       (default "DEFAULTCONFIGFILENAME")\n"
"-c, --camera NAME      camera name NAME\n"
"-p, --port N           port number N (0xNNN - hex, NNN - decimal)\n"
"--porttype TYPE        port type TYPE (unknown, unidirectional\n"
"                       or bidirectional)\n"
"-b, --brightness N     brightness N (0-255)\n"
"--contrast N           contrast N (0-255)\n"
"--white N              white level N (0-255)\n"
"-w, --blackoffset N    black offset N (0-255)\n"
"--saturation N         saturation N (0-255)\n"
"--hue N                hue N (0-255)\n"
"--black N              black level N (0-255)\n"
"-l, --xoffset N        left image edge offset (0-326)\n"
"-t, --yoffset N        top image edge offset (0-326)\n"
"-x, --width N          image width (2-328)\n"
"-y, --height N         image height (2-328)\n"
"-B, --bpp N            bits per pixel (4, 6 or 24)\n"
"-s, --transfer N       transfer scale (1, 2 or 4)\n"
"--direction DIRECTION  camera direction (up, down, right or left)\n"
"-v, --verbose          verbose mode\n"
"-h, --help             this message\n"
;

int lastx,lasty,lastxoffset,lastyoffset,lasttransfer;

/* qcread: Initalize the camera.
   input: amount of arguments, command line arguments. 
   output: error value.
*/
int qcread_init(int width, int height, int xoffset, int yoffset, int transfer){

int option_index,curropt,verboseflag=0;
char *configfilename=NULL,*cameraname=NULL;

int bidiroverride=0, blackoverride=0, bppoverride=0, brightnessoverride=0,
 contrastoverride=0, directionoverride=0, ysizeoverride=0, hueoverride=0,
 portoverride=0, saturationoverride=0, decimoverride=0, whiteoverride=0,
 blackoffsetoverride=0, xsizeoverride=0, xoverride=0, yoverride=0;

int bidirvalue=0, blackvalue=0, bppvalue=0, brightnessvalue=0,
 contrastvalue=0, directionvalue=0, ysizevalue=0, huevalue=0,
 portvalue=0, saturationvalue=0, decimvalue=0, whitevalue=0,
 blackoffsetvalue=0, xsizevalue=0, xvalue=0, yvalue=0;


struct option longoptions[]={
 {"version",0,0,0},
 {"config",1,0,0},
 {"camera",1,0,0},
 {"port",1,0,0},
 {"porttype",1,0,0},
 {"brightness",1,0,0},
 {"contrast",1,0,0},
 {"white",1,0,0},
 {"blackoffset",1,0,0},
 {"saturation",1,0,0},
 {"hue",1,0,0},
 {"black",1,0,0},
 {"yoffset",1,0,0},
 {"xoffset",1,0,0},
 {"height",1,0,0},
 {"width",1,0,0},
 {"bpp",1,0,0},
 {"transfer",1,0,0},
 {"direction",1,0,0},
 {"verbose",0,0,0},
 {"help",0,0,0},
 {0,0,0,0}
 };



printf("Getting Camera\n");

printf("TRANSFER PASSED: %d\n",transfer);
printf("WIDTH PASSED: %d\n",width);
printf("Height PASSED: %d\n",height);

qc=qccreate();
if(!qc){
 fputs("qcread: Insufficient memory\n",stderr);
 exit(1);
 }

printf("Got Camera\n");


qcreadconfigfile(qc,configfilename,cameraname,0);

if(width) {
	qc->xsize=width;
	lastx=width;
}
if(height) {
	qc->ysize=height;
	lasty=height;
	
}


qc->x=xoffset;
lastxoffset=xoffset;
qc->y=yoffset;
lastyoffset=yoffset;

if(transfer)
	qc->decim=transfer;
lasttransfer=qc->decim;

/*
if(portoverride){
 qc->p0=portvalue;
 qc->p1=portvalue+1;
 qc->p2=portvalue+2;
 }
if(bidiroverride) qc->bidir=bidirvalue;
if(blackoverride) qc->black=blackvalue;
if(bppoverride) qc->bpp=bppvalue;
if(brightnessoverride) qc->brightness=brightnessvalue;
if(contrastoverride) qc->contrast=contrastvalue;
if(directionoverride) qc->direction=directionvalue;
if(ysizeoverride) qc->ysize=ysizevalue;
if(hueoverride) qc->hue=huevalue;
if(saturationoverride) qc->saturation=saturationvalue;
if(decimoverride) qc->decim=decimvalue;
if(whiteoverride) qc->white=whitevalue;
if(blackoffsetoverride) qc->blackoffset=blackoffsetvalue;
if(xsizeoverride) qc->xsize=xsizevalue;
*/

printf("width %d, height %d ",qc->xsize/qc->decim,qc->ysize/qc->decim);
printf("xoffset %d, yoffset %d\n",qc->x/qc->decim,qc->y/qc->decim);
printf("transfer %d %d\n",qc->decim, transfer);


printf("Setting up options for camera.\n");

switch(qcinitset(qc)){

 case -1:
  fputs("qcread: Initialization failed: Can't run as non-root\n",stderr);
  exit(1);
 case -2:
  perror("qcread: Initialization failed");
  exit(1);
 case -3:
  fputs("qcread: Initialization failed: QuickCam not detected\n",stderr);
  exit(1);
 case -4:
  printf("qcread: Initialization failed: QuickCam not detected or unsupported type\n");
  exit(1);
 case -5:
  fputs("qcread: Initialization failed: QuickCam doesn't work\n",stderr);
  exit(1);
 case -6:
  fputs("qcread: Image setup failed\n",stderr);
  exit(1);
 case -7:
  fputs("qcread: Invalid frame parameters\n",stderr);
  exit(1);
 case -8:
  fputs("qcread: Frame setup failed\n",stderr);
  exit(1);
 case -9:
  fputs("qcread: Frame memory allocation failed\n",stderr);
  exit(1);
 case 0: 
  if(verboseflag){
   printf("%sdirectional parallel port at 0x%03x with %s Quickcam camera\n",
    qc->bidir==2?"Bi":"Uni",qc->p0,(qc->camv==QCAM_VER_COLOR)?"color":"b&w");
   }
  break;
 default: 
  printf("qcread: Unknown error\n");
  exit(1);
  }
if(verboseflag) fputs("qcread: Reading frame\n",stderr);

printf("Camera setup complete\n");

return 0;
}

/* qcread_image: reads an image from the camera.
   input: location to store image.
   ouput: location contains the image.
*/
void qcread_image(unsigned char *scan){

	int error;
	
/* Have the camera store the image in the buffer. */	
qc->frame1=scan;


/* Take a picture, and if an error happens reinitalize the camera. */
if((error=qcreadframe(qc))){

	printf("Error %d, going to fix camera\n",error);
	qcdelete(qc);
	qcread_init(lastx,lasty,lastxoffset,lastyoffset,lasttransfer);		
}
 

/* I used to need to copy the frame, because I pointed to the wrong part of the camera structure. */
//bcopy(qc->frame,scan,qc->framesize);

}


/* qcread_framesize: return the frame size. */
long qcread_framesize(){
	return qc->framesize;
}

/* qcread_imagetype: return the transfer size. */
int qcread_imagetype(){
	return qc->decim;
}

/* qcread_x: return the x demention. */
int qcread_x() {
	return qc->xsize/qc->decim;
}	

/* qcread_y: return the y demention. */
int qcread_y() {
	return qc->ysize/qc->decim;
}

/* qcread_zoom: return the zoom size. */
int qcread_zoom() {
	return qc->zoom;
}

/* qcread_buffer_size: return the amount of buffers. */
int qcread_buffer_size() {
	return qc->buffers;
}

/* qcread_yield: return the yield size. */
int qcread_yield() {
	return qc->yield;
}

/* qcread_display_rate: return the display every nth frame. */
int qcread_display_rate() {
	return qc->display_rate;
}

/* qcread_xoffset: return the x offset of the frame. */
int qcread_xoffset() {
	return qc->x;
}

/* qcread_yoffset: return the y offset of the frame. */
int qcread_yoffset() {
	return qc->y;
}

int qcread_brightness() {
	return qc->brightness;
}

int qcread_contrast() {
	return qc->contrast;	
}

/* kill_camera: destroy's the Quick Cam structure and releases the IO. */
void kill_camera() {
	qcdelete(qc);
}

/* print_camera_info: Displays the info from the configuration file. */
void print_camera_info(){
	char *up="UP";
	char *down="DOWN";
	char *left="LEFT";
	char *right="RIGHT";
	char *direction;
	
	char *bidir="BI-DIRECTIONAL";
	char *other="STANDARD";
	char *port_type;
	printf("\n\nCAMERA INFORMATION:\n\n");
	printf("Image Offset: %d , %d\n",qc->x,qc->y);
	printf("Image Size: %d x %d (before transfer)\n",qc->xsize,qc->ysize);
	printf("Transfer Size: %d\n", qc->decim);
	printf("Image Size: %d x %d (after transfer)\n",qcread_x(),qcread_y());
	switch(qc->direction) {
		case 0: direction=up; break;
		case 1: direction=right; break;
		case 2: direction=down; break;
		case 3: direction=left; break;
		default: direction="error"; break;
	}
	printf("Direction of Camera: %s\n",direction);
	printf("bpp: %d\n",qc->bpp);
	printf("Brightness: %d\n",qc->brightness);
	printf("Contrast: %d\n",qc->contrast);
	printf("Hue: %d (in color Qcam only) \n",qc->hue);
	printf("Saturation: %d\n",qc->saturation);
	printf("Black Level (for color Qcam only: %d\n",qc->black);
	printf("While Level (for color Qcam only: %d\n",qc->white);
	printf("Black Offset ""white balance"" (for B/W Qcam only): %d\n",
		qc->blackoffset);
		
	switch(qc->bidir) {
		case 2: port_type=bidir; break;
		default: port_type=other;
	}
	printf("Port Type: %s\n",port_type);
	printf("I/O Port Handle (if used): %d\n",qc->ioporthandle);
/*	printf("Zoom: x%d\n",qc->zoom);
	printf("Image Size After Zoom: %d * %d\n",
		qcread_x()*qc->zoom,qcread_y()*qc->zoom);
	printf("Buffer Size: %d\n",qc->buffers);	
	printf("Yield: %d\n",qc->yield);
	printf("Display Every %dth Frame.\n",qc->display_rate);
*/
	printf("\n");
}
