package pinball.part;

import pinball.*;
import pinball.io.*;
import pinball.buffer.*; 
import pinball.server.*;
import pinball.player.*;

import java.io.*;
import java.util.*;

/**
 * Used to detect that the ball has gone out of play. 
 */
public class BallReturn {
	/** Used to control the ballReturn. */
	private static transient DataIO drain;
			
	/** Is the ball in the drain. */
	private boolean ballIsInDrain=false;
	
	/** Data from the port. */
	private byte data[];
	
	/** Code to detect ball's in the drain. */
	private final int BALLINDRAIN = 1;
	
	/** Code used to return the ball. */
	private final int RETURNBALL = 4;

	/** Is the ball return waiting for a read to have taken place. */
	private boolean awaitingResult=false;
	
	/** The last time the ball return polled the machine. */
	private long lastTimePoll=0; 
	
	/** Number of milliseconds between polls. */
	private long pollTime=1000;

	/** 
	 * Controls the ball return of the pinball machine.
	 *
	 * @param dataIO the device to talk to for controlling the drain.
	 */
	public BallReturn(DataIO dataIO){
		/* If playing from a file, don't make the ball return. */
		if(GameOptions.play())
			return;
			
		drain=dataIO;
		
		System.err.println("BallReturn Made.");
		
	}
	
	/**
	 * Make the ball return plunger go back down. 
	 */
	public void reset() {
		try{drain.resetOutputLine(RETURNBALL);}
		catch(Exception e){}
	}

	/**
	 * Checks if the ball is in the drain.
	 *
	 * @param return ball in drain status.
	 */
	public boolean ballInDrain() {
		if(System.currentTimeMillis()-lastTimePoll<=pollTime) {
			return false;
		}
		
		try{	
			lastTimePoll=System.currentTimeMillis();
			/* If the system is waiting from a previous read. */
			if(!awaitingResult)
				drain.requestRead();

			awaitingResult=true;

			if(drain.ReadInput(BALLINDRAIN))
				ballIsInDrain=true;
			else ballIsInDrain=false;
			
			/* If it made it this far, then the read went ok. */
			awaitingResult=false;
		}
		catch (Exception e){
			//System.err.println("Hmm... "+awaitingResult);
			ballIsInDrain=false; 			
		}
		
		return ballIsInDrain;
	}
}

