package pinball.part;

import java.io.*;
import java.awt.*;

/**
 * These are the objects found in the image. It contains where to object was
 * found and it's size.
 */
public class Blip implements Serializable{
	/* Used to keep the same serialized version. */
	static final long serialVersionUID = -8800238146397209048L;

	/** X location of ball. */
	protected int x=-1;

	/** Y location of ball. */
	protected int y=-1;
	
	/** Last X location ball was found. */
	protected int foundX=-1;
	
	/** Last Y location ball was found. */
	protected int foundY=-1;

	/** Total x values for the blip. */
	protected int totalX=0;
	
	/** Total y values for the blip. */
	protected int totalY=0;
	
	/** Total pixels it takes up. */
	protected int total=0;
	
	/** Max x value for the blip. */
	protected int maxX=-1;
	
	/** Max y value for the blip. */
	protected int maxY=-1;

	/** Min x value for the blip. */
 	protected int minX=999999;
	
	/** Min y value for the blip. */
	protected int minY=999999;
	
	/** The alias for the blip, which blip is it part of. */
	protected int alias=-1;
	
	/** End Points. */
	protected Point start, end;
	
	/** 
	 * Create blip with the location seen. 
	 *
	 * @param x the x location.
	 * @param y the y location.
	 */
	public Blip(int x,int y) {
		x(x);
		y(y);		
	}
	
	/** 
	 * Create blip with the location seen. 
	 *
	 * @param x the x location.
	 * @param y the y location.
 	 * @param total the total blips used by the blip.
	 */
	public Blip(int x,int y, int total) {
		x(x);
		y(y);
 		this.total=total;	
	}
	
	
	/**
	 * Create a blank blip. 
	 */
	public Blip() {}
	
	/**
	 * Merge two blips together.
	 *
	 * @param merger the blip to merge with. 
	 */
	public void merge(Blip merger) {
		/* Don't merge two of the same blip. */
		if(merger.alias() == this.alias) {
			return;
		}
		totalX+=merger.totalX();
		totalY+=merger.totalY();
		total+=merger.total();

		if(merger.minX()<minX)
			minX=merger.minX(); 
		if(merger.minY()<minY)
			minY=merger.minY();
		if(merger.maxX()>maxX)
			maxX=merger.maxX();
		if(merger.maxY()>maxY)
			maxY=merger.maxY();
			
		//System.err.println("Merging "+this.alias+" & "+alias);
		
		int tempAlias=merger.alias();
		
		merger.alias(this.alias);
		
		this.alias=tempAlias;
		
		
	}
			
	/**
	 * Set the alias for this blip. 
	 *
	 * @param a the alias for this blip.
	 */		
	public void alias(int a) {
		alias=a;
	}
	
	/**
	 * Get the alias for this blip.
	 *
	 * @return blip's alias.
	 */
	public int alias() {
		return alias;
	}
	
	/** 
	 * Set the start location. 
	 *
	 * @param x the x location
	 * @param y the y location
	 */
	public void start(int x, int y) {
		/* If the starting point is already set, don't reset it. 
		   This applies to vertical lines.	
		*/
		if(start!=null && start.x==x)
			return;	
		else start=new Point(x,y);
	}
	
	/** 
	 * Get the start location. 
	 * @return the start location.
	 */
	public Point start() {
		return start;
	}
	
	/** 
	 * Set the end location. 
	 *
	 * @param x the x location
	 * @param y the y location
	 */
	public void end(int x, int y) {
		end=new Point(x,y);
	}

	/** 
	 * Get the end location. 
	 * @return the end location.
	 */
	public Point end() {
		return end;
	}

	/**  
	 * Add to the x location, find the min & max x, and increment the 
	 * total pixels used up by this blip.
	 *
	 * @param x the x location found.
	 */
	public void xinc(int x) {
		if(x<minX)
			minX=x;
		if(x>maxX)
			maxX=x;
		totalX+=x;
		total++;
	}

	/**  
	 * Add to the y location, find the min & max y.
	 *
	 * @param y the y location found.
	 */
	public void yinc(int y) {
		if(y<minY)
			minY=y;
		if(y>maxY)
			maxY=y;
		totalY+=y;
	}	

	/**
	 * Get the total pixels used by this blip.
	 *
	 * @return The total about of pixels used by this blip.
	 */
	public int total() {
		return total;
	}
		

	/**
	 * Set the x location. 
	 *
	 * @param x the location.
	 */
	public void x(int x) {
		this.x=x;
	}

	/**
	 * Set the y location. 
	 *
	 * @param y the location.
	 */
	public void y(int y) {
		this.y=y;
	}
		
	
	/** 
	 * Get the total x. 
	 * 
	 * @return the total x values for all the pixel locations this blips occupies.
	 */
	public int totalX() {
		return totalX;
	}
	

	/** 
	 * Get the total y. 
	 * 
	 * @return the total y values for all the pixel locations this blips occupies.
	 */
	public int totalY() {
		return totalY;
	}	
	

	/** 
	 * Get the x location of this blip. 
	 *
	 * @returns x location.
	 */
	public int xLocation() {
		if(total!=0)
			return totalX/total;
		else return -1;
	}
	
	/** 
	 * Get the y location of this blip. 
	 *
	 * @returns y location.
	 */
	public int yLocation() {
		if(total!=0)
			return totalY/total;
		else return -1;
	}	

			
	/** 
	 * Get the x location of this blip. 
	 *
	 * @returns x location.
	 */
	public int x() {
		return x;
	}
	
	/** 
	 * Get the x location of this blip. 
	 *
	 * @returns x location.
	 */
	public int y() {
		return y;
	}	
	
	/**
	 * Last found x location of blip. 
	 *
	 * @return last x location.
	 */
	public int foundX() {
		return foundX;
	}

	/**
	 * Last found y location of blip. 
	 *
	 * @return last y location.
	 */	
	public int foundY() {
		return foundY;
	}
	
	/**
	 * minimum x location of blip. 
	 *
	 * @return minimum x location.
	 */
	public int minX() {
		return minX;
	}
	
	/**
	 * minimum y location of blip. 
	 *
	 * @return minimum y location.
	 */	
	public int minY() {
		return minY;
	}
	
	/**
	 * maximum x location of blip. 
	 *
	 * @return maximum x location.
	 */	
	public int maxX() {
		return maxX;
	}

	/**
	 * maximum y location of blip. 
	 *
	 * @return maximum y location.
	 */		
	public int maxY() {
		return maxY;
	}

	/**
	 * Get the width of the blip.
	 *
 	 * @return width.
	 */
	public int width() {
		return (maxX-minX);
	}

	/**
	 * Get the height of the blip.
	 *
 	 * @return height.
	 */
	public int height() {
		return (maxY-minY);
	}

	/** Save the object to disk. 
	* @param s the stream to write to.
	*/
	protected void writeObject(ObjectOutputStream s)
                                           throws IOException {
                 	System.err.flush();
                        foundX=x;
                        foundY=y;
                        s.defaultWriteObject();
	}

	/** Read object from disk.
	 * @param the stream to read from.
	 */
	protected void readObject(ObjectInputStream s)
                                          throws IOException  {
                                          
                      try {s.defaultReadObject();
                      }
  		      catch (ClassNotFoundException e){System.err.println(e);}
  		      	

        }
	
}



