package pinball.part;

import java.awt.*;
import java.io.*;

/** Control the car so it can run. */
public class CarRemote extends Frame implements Runnable{
	/** Lock the car, so only 1 instance of the car can be used. */
	private FileOutputStream lock;
	private String lockName="/tmp/car.lock";
	
	private Button quit;
	private Checkbox power;
	private Scrollbar speed;
	private Car controller;
	private boolean running=true;
	private Thread remote;
	
	public static void main(String args[]) {
		new CarRemote();
	}
	
	public CarRemote() {
		/* Lock is found, do not allow other instances to run. */
		if(new File(lockName).exists()){
			System.err.println("Only one instance of CarRemote is allowed.");
			System.err.println("If you need to, remove the lock file: "+lockName);
			System.exit(0);
		}

		try {
			lock = new FileOutputStream(lockName);
			lock.write(1);
			lock.close();
		}
		catch (Exception e){}
		
		setLayout(new GridLayout(3,1,5,0));
		setSize(300,200);
		setTitle("Racing Car");
		add(power=new Checkbox("Power"));
		add(speed=new Scrollbar(Scrollbar.HORIZONTAL,0,1,0,3));
		add(quit=new Button("Quit"));
		setVisible(true);
		controller=new Car();
		remote=new Thread(this);
		remote.start();
	}
	
	public void stop(){running=false;}
	
	public void run() {
		int current;
		while(running) {
			try { 
			if(power.getState() && running)
				controller.on();
			current=speed.getValue();				
			if(current==0)
				Thread.sleep(150);
			if(current==1)
				Thread.sleep(300);
			if(current==2)
				Thread.sleep(1000);
			controller.off();
			Thread.sleep(50);				
			}
		
			catch(Exception e){System.err.println(e);} 			
		}	

	}
		
	public void releaseLock() {
		new File(lockName).delete();
	}

	public boolean action(Event e, Object arg) {
		Object target = e.target;
		/* When the user presses quit exit. */
	 	if(target==quit) {
			try{
				controller.off();
				Thread.sleep(50);
				running=false;
				releaseLock();
				System.exit(0);
			}
			catch(Exception f){System.err.println(f);}
			return true;
		}
		return false;
	}
	
}
