package pinball.part;

import java.awt.*;
import java.io.*;

import pinball.io.*;

/** Used to flash a light connected to the computer. */
public class Flasher extends Frame{
	/** The controller. */
	private static DataIO light;

	/** The button to press to quit. */
	private Button quit;
	
	/** Is the system exiting. */
	private boolean exiting=false;
	
	/** The lock file. */
	private FileOutputStream lock;

	/** The lock file name. */
	private String lockName="/tmp/flasher.lock";
	
	/** Which relay is the light on. */
	private int relay=4;
	
	/** Test the light. */
	public static void main(String args[]) {
		new Flasher().flash();
		
	}
	
	/** Setup the light. */
	public Flasher() {
		/* Lock is found, do not allow other instances to run. */
		if(new File(lockName).exists()){
			System.err.println("Only one instance of Flasher is allowed.");
			System.err.println("If you need to, remove the lock file: "+lockName);
			System.exit(0);
		}

		try {
			lock = new FileOutputStream(lockName);
			lock.write(1);
			lock.close();
		}
		catch (Exception e){}
			
		light=new DataIO();	
	}
	
	/** Open a window saying that mail has arrived and flash the light. */
	private void flash(){
		setSize(100,50);
		setTitle("You've got mail!");
		quit=new Button();
		quit.setLabel("quit");
		try{
			setVisible(true);
		}
		catch(Exception g) {new File(lockName).delete();}
		add(quit);
		while(true) {
			try { 
				if(!exiting)
					light.setOutputLine(relay);
				Thread.sleep(500);
				light.resetOutputLine(relay);			
				Thread.sleep(3000);
			}
			catch(Exception e){System.err.println(e);}
		}
	}
	
	/** The events for the window. When OK is pressed, continue with the program.*/
	public boolean action(Event e, Object arg) {
		Object target = e.target;
		/* When the user presses quit exit. */
	 	if(target==quit) {
			try{
				exiting=true;
				light.resetOutputLine(relay);
				Thread.sleep(500);
				new File(lockName).delete();
				System.exit(0);
			}
			catch(Exception f){System.err.println(f);}
			return true;
		}
		return false;
	}
	
	/** The system will release the lock. */
	public void releaseLock() {
		new File(lockName).delete();
	}
	
	/** Turn the light on. */
	public void on() {
		try{
		light.setOutputLine(relay);
		}
		catch(Exception e){}
	}
	
	/** Turn the light off. */
	public void off() {
		try{light.resetOutputLine(relay);}
		catch(Exception e){}
	}

}
