package pinball.part;

import pinball.*;
import pinball.io.*;


import java.io.*;
import java.util.*;

/** The flippers in the pinball machine. */
public class Flipper implements Serializable{
	/** Used to keep the same serialized version. */ 
	static final long serialVersionUID = 1065289741869851849L;

	/** Used to control the flipper. */
	private transient DataIO flip;
	
	/** Which type of flipper it is, left or right. */
	private int which;
	
	/* Definitions for the flippers. */

	/** Left flipper. */
	public static  int LEFT=0;
	
	/** Right flipper. */	
	public static  int RIGHT=2;
		
	/** X location of the flipper. */
	private int x=0;
	
	/** Y location of the flipper. */
	private int y=0;
	
	/** Offset of flipper from top of frame. */
	private int yOffset=0;
	
	/** Is the flipper down? */
	private boolean down=true;
	
	/** The time since last flipped. */
	private FlipperTimeStamp timeStamp;
	

	/** 
	 * Set up the flipper to the left or right. 
	 *
	 * @param which Which flipper this is. 0 = left, right = 1.
	 */
	public Flipper(DataIO flip,int whichFlipper) {
		this.flip=flip;
		
		
		/* Set the left flipper. */
		if(whichFlipper==LEFT) 
			which=LEFT;	
			
		/* Set the right flipper. */
		else if(whichFlipper==RIGHT) 
			which=RIGHT;	
		
		/* Invalid flipper assignment. */
		else {
			System.err.println("Wrong flipper assignment!");
			System.exit(1);
		}				

		timeStamp= new FlipperTimeStamp();
		
		System.err.println("Flipper made.");
		
		/* Make the flipper go down. */
		reset();
	}
			
	/**
	 * Set the y location of the flipper. 
	 *
	 * @param x The flipper's x location.
	 */			
	public void x(int x) {
		this.x=x;
	}

	/**
	 * Set the y location of the flipper. 
	 *
	 * @param y The flipper's y location.
	 */			
	public void y(int y) {
		this.y=y;
	}
	
	/**
	 * Get the x location of the flipper.	  
	 *
	 * @return x location of flipper.
	 */
	public int x() {
		return x;
	}
	
	/**
	 * Get the y location of the flipper.	  
	 *
	 * @return y location of flipper.
	 */
	public int y() {
		return y;
	}
	

	/**
	 * Set the Y offset of the flipper. 
	 *
	 * @param y y offset of flipper.
	 */
	public void yOffset(int y) {
		this.yOffset=y;
	}
	
	
	/** 
	 * Get the Y offset of the flipper. 
	 *
	 * @return y offset of flipper.
	 */
	public int yOffset() {
		return yOffset;
	}


	/** 
	 * Do a flip, but if already up do not flip. 
	 */
	public void flip() {
		/* If the flipper is down, flip, if it's up don't flip since it is already up. */
		if(down==true) {
			/* Flip the flipper. */
			try	{flip.setOutputLine(which);}
			catch (IOException e){
					System.err.println("DEVICE WRITE ERROR: "+e);
					//System.exit(1);
			}		
			
		}

		/* Reset the reset timer, so the flipper can still stay up. */		
		timeStamp.flip();

		/* Flipper is now up. */
		down=false;
		
		//System.err.println("Requested flip");	

	}

	/**
	 * Reset the flipper. 
	 */
	public void reset() {	
		/* Reset the flipper. */
		try	{
			flip.resetOutputLine(which);
		}
		catch (IOException e){
				System.err.println("DEVICE WRITE ERROR: "+e);
				//System.exit(1);
		}
		/* Flipper is down. */
		down=true;
	}
	
	/** 
	 * Get the time stamp for the flipper.
	 *
	 * @return timeStamp 
	 */
	public FlipperTimeStamp timeStamp() {
		return timeStamp;
	}
	
	/**
	 * Is the flipper up.
	 *
	 * @return flipper's up status.
	 */
	public boolean up() {
		return !down;
	}
	
	/**
	 * Is the flipper down.
	 *
	 * @return flipper's down status. 
	 */
	public boolean down() {
		return down;
	}
	
	/** Write the flipper to disk. 
	* @param s stream to write to.
	*/
	private void writeObject(ObjectOutputStream s)
                                           throws IOException {
                      s.defaultWriteObject();
                      
                      // customized serialization code
                      
                      s.writeInt(LEFT);
                      s.writeInt(RIGHT);
        }

	/* Read the flipper from the disk. 	
	* @param s stream to read from.
	*/

	private void readObject(ObjectInputStream s)
                                          throws IOException  {
                      try {s.defaultReadObject();

                      // customized deserialization code

		      LEFT=(int)s.readInt();
		      RIGHT=(int)s.readInt();
		      }
  		      catch (ClassNotFoundException e){System.err.println(e);}                    
                    
                   		
		
		reset();
		
        }
}
