package pinball.part;

import java.awt.*;
import java.io.*;

/** Control the robot by the keyboard. */
public class MovitRobotRemote extends Frame implements Runnable{
	/** Lock, so only 1 can control the robot. */
	private FileOutputStream lock;
	
	/** Lockfile name. */
	private String lockName="/tmp/movitrobot.lock";
	
	/** Quit the system. */	
	private Button quit;

	/** Controls the robot. */
	private MovitRobot controller;
	
	/** Is the system running. */
	private boolean running=true;
	
	/** System thread. */
	private Thread remote;
	
	/** States of the robot. */
	public static int FORWARD=1, BACKWARD=2, LEFT=3, RIGHT=4, STOPPED=0;

	/** Current state. */
	private int state=STOPPED;
	
	/** Time that a command was executed. */
	private long lastTime=System.currentTimeMillis();
	
	/** Start up the system. *.
	public static void main(String args[]) {
		new MovitRobotRemote();
	}
	
	/** Place a Quit button on the screen and take the arrow keys to
	control the robot. */
	public MovitRobotRemote() {
		/* Lock is found, do not allow other instances to run. */
		if(new File(lockName).exists()){
			System.err.println("Only one instance of MovitRobotRemote is allowed.");
			System.err.println("If you need to, remove the lock file: "+lockName);
			System.exit(0);
		}

		try {
			lock = new FileOutputStream(lockName);
			lock.write(1);
			lock.close();
		}
		catch (Exception e){}
		
		setLayout(new GridLayout(1,2,5,0));
		setSize(200,200);
		setTitle("Movit Robot");
		add(quit=new Button("Quit"));
		setVisible(true);
		controller=new MovitRobot();
		remote=new Thread(this);
		remote.start();
	}
	
	/** Stop the system. */
	public void stop(){running=false;}
	
	/** Poll the system to see if the user was pressing the keyboard.
	If no key was pressed for a while, stop the robot. */
	public void run() {
		while(true){
			try {
				Thread.sleep(50);
				long time=System.currentTimeMillis();
				if(state==STOPPED && time-lastTime>50)
					controller.stop();
			}
			catch (Exception e){}
		}
	}
		
	/** The current state. 
	@return the state.
	*/
	public int state() {
		return state;
	}
	
	/** Delete the lock file. */
	public void releaseLock() {
		new File(lockName).delete();
	}
	
	/** The user uses the arrow keys on the keyboard to control the robot.
	*/
	public boolean handleEvent(Event event) {
  		if (event.id == Event.KEY_ACTION) {
			if(event.key == Event.UP){
				if(state!=FORWARD)
					controller.forward();
				state=FORWARD;
				return true;
			}

			if(event.key == Event.DOWN){
				if(state!=BACKWARD)
					controller.backward();
				state=BACKWARD;
				return true;
			}

			if(event.key == Event.LEFT){
				if(state!=LEFT)
					controller.counterClockwise();
				state=LEFT;
				return true;
			}
	
			if(event.key == Event.RIGHT){
				if(state!=RIGHT)
			 		controller.clockwise();
				state=RIGHT;
				return true;
			}									
		}		
		if (event.id == Event.KEY_PRESS) {
			if(event.key=='q' || event.key=='Q') {
				quit();
				return true;
			}
		}	
		if (event.id == event.KEY_ACTION_RELEASE) {
//			controller.stop();
			state=STOPPED;
			lastTime=System.currentTimeMillis();			
		}			
		return super.handleEvent(event);
	}
  
  	/** When the user presses the Quit button, exit the system. */
	public boolean action(Event e, Object arg) {
		Object target = e.target;
		/* When the user presses quit exit. */
	 	if(target==quit) {
			quit();
			return true;			
		}
		return false;
	}

	/** Exits the system. */
	public void quit() {	
		try{
			controller.off();
			Thread.sleep(50);
			running=false;
			releaseLock();
			System.exit(0);
		}
		catch(Exception f){System.err.println(f);}
	}	
}
