package pinball.part;

import java.io.*;
import java.awt.*;

import pinball.buffer.*;
import pinball.*;
import pinball.log.*;

/** Draw the sound graphs on the screen. */
public class SoundCardGraph extends Canvas {
	/** The sound sample from the card. */
	public byte sample[];

	/** Setup the canvas.
	 * @param width the width of the canvas
	 * @param height the height of the canvas.
	 */		
	public SoundCardGraph(int width, int height) {
		setSize(width,height);
	}

	/** Set the sample.
	 * @param sample Sound sample.
	 */
	public void setSample(byte sample[]) {
		this.sample=sample;
		repaint();
	}

	/** Update the display. 
	 * @param g Graphics context.
	 */
	public void update(Graphics g) {
		paint(g);
	}

	/** Draw the sound graph. 
	 * @param g Graphics context.
  	 */
	public void paint(Graphics g) {
		g.setColor(Color.cyan);
		g.fillRect(0,0,size().width,size().height);
		g.setColor(Color.black);
		g.drawLine(0,128,size().width,128);
		g.setColor(Color.red);
		
		int lastX=0,lastY=0;
		for(int i=0; i<sample.length; i++) {

			if(sample[i]>=0) {
				g.drawLine(lastX,lastY,i,127-(127-sample[i]));
				lastX=i;
				lastY=127-(127-sample[i]);
			}
			if(sample[i]<0) {
				g.drawLine(lastX,lastY,i,255+sample[i]);
				lastX=i;
				lastY=255+sample[i];
			}

		}
	}
}
