package pinball.player;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;


/** 
 * The analyzed thread. Used to analyze the images from the camera.
 * If finds objects in the frame.
 */
public class Analyze extends Thread {
	/** The information buffer. */
	protected Buffer buffer;
	
	/** The contents of the buffer. */
	protected BufferInfo buffinfo;

	/** The ball to be located. */
	protected Blip ball;
	
	/** The width of the image. */
	protected int xsize;
	
	/** The object to process the image and find things.*/
	protected  ProcessImage process; 
	
		
	/** 
	 * Make a new analyze thread.
	 *
	 * @param buffer The first buffer in the series of buffers. 
	 */
	public Analyze (Buffer buffer) {
		/* The first buffer. */
		this.buffer=buffer;

		this.xsize=buffer.info().width();

	}
	
	/**
	 * For extended classes.
	 */
	public Analyze (){}
	
	/**
	 * Get the name of the object.
	 * 
	 * @param name of the object.

	 */
	public String toString() {
		return "Analyze";
	}

	
	/**
	 * Analyze the images for the ball's location. 
	 */	
	public void run() {
	
		/* Get the first frame for comparison to locate the ball. */
		
		/* Get the lock. */
		buffer.lock().analyze(buffer.id());
		
		/* Get the information. */
		buffinfo=this.buffer.info();
				
		/* Start the time stamp. */
		buffinfo.time().setStartAnalyzeTime();

		/* Setup the process image object. */
		setupProcessImage();

		/* End the time stamp. */
		buffinfo.time().setEndAnalyzeTime();
		
		/* Release the lock. */
		buffer.lock().analyzed(buffer.id());		

		/* Get the next buffer. */
		buffer=buffer.next();
		buffinfo=buffer.info();
		
		/* Let another thread go. */
		Thread.yield();
		
		/* Analyze the images. */
		while(true) {				
			
			/* Get the lock. */
			buffer.lock().analyze(buffer.id());
			
			/* Get the buffer info. */
			buffinfo=this.buffer.info();
			
			/* Start the time. */
			buffinfo.time().setStartAnalyzeTime();			
		
			if(!buffer.skipped()) {
				/* Check the image for ball's location. */
				process.process(buffinfo.get(),buffinfo.width(),buffinfo);
			}
			
			
			/* End the time. */
			buffinfo.time().setEndAnalyzeTime();			
			
			/* Release the lock. */
			buffer.lock().analyzed(buffer.id());

			/* Next buffer. */	
			buffer=buffer.next();

			//skip();			

			
  			/* Let another thread go. */
			Thread.yield();
		}
	} 
	
	/**
	 * Setup the process image object
	 */
	protected void setupProcessImage() {
		/* Get the first image, used in compare to find a change. */
		process=new ProcessImage(buffinfo.get(),buffinfo.width());
	}
	
	/** 
	 * Skip to the most recent buffer.
	 */
	protected void skip() {
		int firstChecked=buffer.id();
		
		/* Not skippable. */
		if(buffer.lock().status()!=buffer.lock().grabed)
			return;			
		
		/* Skip frames while the next buffer has a frame that's usable. */
		while((buffer.next().lock().status()==buffer.next().lock().grabed) && (buffer.next().id()!=firstChecked)) {
		
			/* Let this buffer pass through, since a later one is available. */
			
			/* Get the lock. */
			buffer.lock().analyze(buffer.id());
		
			/* Get the buffer info. */
			buffinfo=this.buffer.info();

			buffer.skipped(true);
			PinballLog.writeln(buffinfo.number(),"analyzeSkippingFrame="+buffer.info().number(),1,this);				
			System.err.println("Analyze skipping frame #"+buffinfo.number());
			
			/* Skip this frame. */
			process.skip(buffinfo);
		
			buffer.lock().analyzed(buffer.id());
			buffer=buffer.next();
		}
	}

}
