package pinball.player;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;


/**
 * This is where the picture is displayed to.
 * It does the drawing of the image, ball location, flippers, flipper zone, in play and out
 * of play area.
 */
public class Post extends Thread {

	/** The current information buffer. */
	protected Buffer buffer;	 
	
	/** The information in the buffer. */
	protected BufferInfo buffinfo;

	/** Frame from for image display. */
	protected int framerate=GameOptions.frameRate();

	/** Area to draw to. */
	protected DrawingCanvas canvas;
	

	/** Stub from the strategy. 
	Used for drawing lines on the screen for analysis. */	
	protected Stub stub;

	/** 
	 * Initalize the DrawingBoard. 
	 *
	 * @param buffer the first in the series of infomation buffers.
	 */
	public Post(Buffer buffer){	
		/* The first buffer. */
		this.buffer=buffer;
		
		setName("Post");	      	
	}
	
	/**
	 * Used in the extended classed. 
	 */
	public Post() {}
	
	/**
	 * Get the name of the object.
	 * 
	 * @param name of the object.
	 */
	public String toString() {
		return "Post";
	}
	
	
	/**
	 * Create the canvas drawn to.
	 *
	 * @param x Width of canvas.
	 *
	 * @param y Height of canvas.
	 *
	 * @return the canvas to drawn on.
	 */
	public DrawingCanvas canvas(int x,int y) {
		this.canvas=new DrawingCanvas(x,y);
			      	
		return this.canvas;
	}
	/**
	 * Get the canvas drawn to.
	 *
	 * @return the canvas drawn to.
	 */	
	public DrawingCanvas getCanvas() {
		return canvas;
	}

	/**
	 * Keep looking at the images and draw them at the frame rate from GameOptions. 
	 */
	public void run(){
			
		/* If drawing the frames. */
		if(canvas!=null) {
			canvas.makeImage();
		}
		
		/* Setup the stub in the DrawingCanvas. */
		setupStub();	
		
		/* Keep drawing the images on the canvas. */	
		while(true) {

			buffer.lock().post(buffer.id());
			buffinfo=this.buffer.info();			
			
			buffinfo.time().setStartDisplayTime();
			
			if(!buffer.skipped()) {						
				/* display the image, if it's the frame of the frame rate. */				
		        	if(buffinfo.number()%framerate==0) {
					/* Draw image. */
					if(GameOptions.debug()>=2)
						System.err.println("Frame #"+buffinfo.number());
				
					if(canvas!=null)
						canvas.updateFrame(buffinfo); 
								
	
					if(GameOptions.debug()>=2)
				     		System.err.println("Going to paint.");	
				}		
			}	

			buffinfo.time().setEndDisplayTime();

			buffer.lock().posted(buffer.id());						
			
			buffer=buffer.next();
			
			//skip();
		
			Thread.yield();
		}

	}

	/**
	 * Setup the stub in the drawing canvas. 
	 */
	public void setupStub() {
		buffer.lock().post(buffer.id());
		buffinfo=this.buffer.info();	
		
		canvas.setStub(buffinfo.getStub());
			
		buffer.lock().posted(buffer.id());						 	
		buffer=buffer.next();
	
		Thread.yield();
	}
		
			
		

	/** 
	 * Skip to the most recent buffer.
	 */
	private void skip() {
		int firstChecked=buffer.id();
				
		/* Not skippable. */
		if(buffer.lock().status()!=buffer.lock().strategied)
			return;			 
		
		/* Skip frames while the next buffer has a frame that's usable. */
		while((buffer.next().lock().status()==buffer.next().lock().strategied) && (buffer.next().id()!=firstChecked)) {

			/* Let this buffer pass through, since a later one is available. */

			/* Get the lock. */
			buffer.lock().post(buffer.id());
		
			/* Get the buffer info. */
			buffinfo=this.buffer.info();

			PinballLog.writeln(buffinfo.number(),"drawingBoardSkippingFrame="+buffer.info().number(),1,this);				
			System.err.println("DrawingBoard skipping frame #"+buffinfo.number());

			buffer.skipped(true);
			
			/* Release Lock. */
			buffer.lock().posted(buffer.id());
			
			buffer=buffer.next();
			
		}		
	}
}
