package pinball.player;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import java.awt.*;

/**
 * Find's the location of the blip in the image. 
 */
public class ProcessImage implements Serializable{
	/** Used to keep the same serialized version. */
	final long serialVersionUID = 793157465111124568L;

	/* constants */
	/** The threshold in comparing the images for changes.*/
	protected  int threshold = GameOptions.pixelThreshold(); 	

	/* Values for the difference images. */
	
	/** A white pixel. */
	protected  byte white=(byte) 63;
	
	/** A black pixel. */
	protected  byte black=0;

	/** The last frame. */
	protected byte[] lastFrame;

	/** The current difference frame. */
	protected byte[] currentDifferenceFrame;
	
	/** Last difference frame. */
	protected byte[] lastDifferenceFrame;
	
	/** Location of blips in the image. */
	protected byte[] blipLocationFrame;
	
	/** Verbose Image. */
	protected byte[] verboseFrame;

	/** Frame of objects. */
 	protected int[][] objectsFrame;

	/** Biggest object. */
	protected int maxObject;
	
	/** Found objects. */
	protected Vector foundObjects;
	
	/** The object working on. */
	protected Blip thing;
		
	/** Used in for loops. */
	protected int i; 	
	
	/** XSize of the image. */
	protected int xsize;
	
	/** YSize of the image. */	
	protected int ysize;
	
	/** Last XSize of the image. */
	protected int lastXSize;
	
	/** Last YSize of the image. */	
	protected int lastYSize;
	
	/** Rate of pixels to skip while processing. */
	protected int pixelRate=GameOptions.pixelRate();
	
	
	/** The first pixel to start at. */
	protected int start;

	/** Size of the last frame. Used to see if the frame has changed size. 
	If so, resize the frames's arrays. 
	*/
	protected int lastFrameSize;
		
	/** 
	 * Make the first frame for comparison. 
	 *
	 * @param frame the frame used for comparison.
	 * @param x the width of the image.
	 */
	protected ProcessImage(byte[] frame, int x) {
		setup(frame,x);
	}
	
	/** For extended classes. */
	protected ProcessImage() {
	}
	
	/** 
	 * Setup the arrays. 
	 *
	 * @param frame the frame used for comparison.
	 * @param x the width of the image.
	 */
	protected void setup(byte[] frame, int x) {
		/* Allocate the frame arrays. */
		lastFrame=new byte[frame.length];
		currentDifferenceFrame=new byte[frame.length];
		lastDifferenceFrame=new byte[frame.length];
		blipLocationFrame=new byte[frame.length];
		verboseFrame=new byte[frame.length];
		objectsFrame=new int[x][frame.length/x];		
		
		foundObjects=new Vector();		
		
		System.err.println("Making ProcessImage");

		/* copy the 1st frame */
		for (i=0; i < frame.length; i++)
			lastFrame[i] = frame[i];			
	
		/* Get the dimentions for the image. */
		xsize=x;
		ysize=frame.length/xsize;
		
		/* Last image size. */
		lastXSize=xsize;
		lastYSize=ysize;				
		
		System.err.println("X="+xsize+"Y="+ysize);
				
		/* Starting location of the first pixel to look at in the image. */		
		start=0;
		
		PinballLog.headerInfo("pixelThreshold="+threshold+" colorDepth="+(white+1),this);
		
		lastFrameSize=frame.length;
	}
	
	
	
	
	/**
	 * Analize the frame for the ball. 
	 *
	 * @param frame frame to analyze
	 * @param width the width of the image
	 * @param buffinfo the storage for the blips it finds.
	 */
	protected void process(byte[] frame, int x, BufferInfo buffinfo) {

		/* Object working on. */
		int currentObject=1;
		
		int xlocation,ylocation;
						
		blipLocationFrame=new byte[frame.length];

		objectsFrame=new int[x][frame.length/x];

		foundObjects=new Vector();		
		
		/* Used in the loop to scan the pixels in the image. These are the ends of the array. */
		int maxX=x-1;
		int maxY=(frame.length/x)-1;

		boolean verbose=GameOptions.verbose();		
				
		int xat,yat;
				
		/* Frame dimentions. */
		xsize=x;
		ysize=frame.length/xsize;
		
		/* If the frame size has changed, make a new arrays. */
		if(lastFrameSize!=frame.length)
			setup(frame,x);
		
		/* Save the last frame's size for next compare. */
		lastFrameSize=frame.length;
				
									
		if(GameOptions.debug()>=1)
			System.err.println("Analyzing frame" +" "+buffinfo.id()+". Start="+start+" End="+frame.length);


		/* Amount pixel has changed. */
		int diff=0;		
		

		/* Reset pixel counter. */
		i=0;
		
		/* Compare the current image to the previous and look for changes.*/

		/* Look at all the pixels in the image. */
		for (ylocation=0; ylocation<= maxY; ylocation+=pixelRate) {			
			for (xlocation=0; xlocation<= maxX; xlocation+=pixelRate) {
				//System.err.println("pixel: "+i+" xlocation "+xlocation+" ylocation "+ylocation)			
						
				/* Get the difference between the pixels. */
				diff=frame[i]-lastFrame[i];		
							
				/* Get the abs of the difference. */
				if(diff<0)
					diff=-diff;
				if (diff >= threshold) {
					currentDifferenceFrame[i] = black;
				}
				else
					currentDifferenceFrame[i] = white;
								
				/* Create the blip location by logically ANDing the last
				   and current difference images. */
				if (lastDifferenceFrame[i] == black &&
				currentDifferenceFrame[i] == black) {				
					blipLocationFrame[i] = black;
					/* Make frame to be the blips image. */
					if(verbose)
						verboseFrame[i]=black;								
				} 
				
				/* Nothing changed. */
				else {
					if(verbose)
						verboseFrame[i]=white;
					blipLocationFrame[i] = white;
				}
				
	
	
				/* Exclusive OR the current difference frame and the blip location frame.
				   If there is a change note it. 
				   This is the new way the finds multiple objects.
				*/
				if (currentDifferenceFrame[i] != blipLocationFrame[i]) {
					
					/* Make frame to be the blips image. */
					//if(verbose)
					//	verboseFrame[i]=black;				

					/* Check to see if other pixels are from this blip. If so, merge or update the objects. */
				
					/* left pixel. */
					if((xlocation-pixelRate)>=0 && objectsFrame[xlocation-pixelRate][ylocation]>0) {
						/* If two connecting pixels are of two different objects merge them. */
						if(objectsFrame[xlocation][ylocation]>0 && objectsFrame[xlocation][ylocation]!=objectsFrame[xlocation-pixelRate][ylocation])
							merge(objectsFrame[xlocation][ylocation],objectsFrame[xlocation-pixelRate][ylocation]);
						objectsFrame[xlocation][ylocation]=objectsFrame[xlocation-pixelRate][ylocation];
					}
					
					/* upper left pixel. */
					if((xlocation-pixelRate)>=0 && (ylocation-pixelRate)>=0 && objectsFrame[xlocation-pixelRate][ylocation-pixelRate]>0) {
						if(objectsFrame[xlocation][ylocation]>0 && objectsFrame[xlocation][ylocation]!=objectsFrame[xlocation-pixelRate][ylocation-pixelRate])
							merge(objectsFrame[xlocation][ylocation],objectsFrame[xlocation][ylocation-pixelRate]);
						objectsFrame[xlocation][ylocation]=objectsFrame[xlocation-pixelRate][ylocation-pixelRate];
					}	
						
					/* upper pixel. */
 					if((ylocation-pixelRate)>=0 && objectsFrame[xlocation][ylocation-pixelRate]>0) {
						if(objectsFrame[xlocation][ylocation]>0 && objectsFrame[xlocation][ylocation]!=objectsFrame[xlocation][ylocation-pixelRate])
							merge(objectsFrame[xlocation][ylocation],objectsFrame[xlocation][ylocation-pixelRate]);
						objectsFrame[xlocation][ylocation]=objectsFrame[xlocation][ylocation-pixelRate];
						
					}
					
					/* upper right. */
					if((xlocation+pixelRate)<=maxX && (ylocation-pixelRate)>=0 && objectsFrame[xlocation+pixelRate][ylocation-pixelRate]>0) {
						if(objectsFrame[xlocation][ylocation]>0 && objectsFrame[xlocation][ylocation]!=objectsFrame[xlocation+pixelRate][ylocation-pixelRate])
								merge(objectsFrame[xlocation][ylocation],objectsFrame[xlocation+pixelRate][ylocation-pixelRate]);
						objectsFrame[xlocation][ylocation]=objectsFrame[xlocation+pixelRate][ylocation-pixelRate];
						
					}
			
					/* New blip found. */
 					if(objectsFrame[xlocation][ylocation]==0) {					
						foundObjects.addElement(new Blip());
						objectsFrame[xlocation][ylocation]=currentObject;
						thing=(Blip)foundObjects.elementAt(objectsFrame[xlocation][ylocation]-1);
						thing.alias(currentObject);				
						currentObject++;
						//System.err.println("An object is found.");
					}
					
	
	 				/* Increment the blip's pixel count. */
					thing=(Blip)foundObjects.elementAt(objectsFrame[xlocation][ylocation]-1);
					thing.xinc(xlocation);
					thing.yinc(ylocation);			
					
				}				
					
						
				/* Go to next pixel. */
				i++;
			}			
		}
		

		/* Update the last image. */
		System.arraycopy(frame,0,lastFrame,0,frame.length);
		

		/* Update the last difference image. */
		System.arraycopy(currentDifferenceFrame,0,lastDifferenceFrame,0,currentDifferenceFrame.length);
		

		if(verbose)
			System.arraycopy(verboseFrame,0,frame,0,verboseFrame.length);

		maxObject=-1;
		
		thing=null;
 
		Vector foundObjects2=new Vector(); 

 		/* Filter out the merged objects. */
		for(int i=0; i<foundObjects.size();i++) {
			thing=(Blip)foundObjects.elementAt(i);
			if(thing.alias()-1==i) {
				foundObjects2.addElement(thing);			
			}
		}
 
		buffinfo.setBlips(objectsFrame,foundObjects2);
	
		lastXSize=xsize;
		lastYSize=ysize;	
		
		if(GameOptions.debug()>=1)
			System.err.println("Frame analyzed"+" "+buffinfo.id());
			
	}
	



	/**
	 * Merge two objects into one. 
	 * 
	 * @param a object a.
 	 * @param b object b.
	 */
	protected void merge(int a, int b) {
		Blip merger1,merger2;		
			
			merger1=(Blip)foundObjects.elementAt(a-1);
			merger2=(Blip)foundObjects.elementAt(b-1);
			
			if(merger1.alias()==merger2.alias())
				return;
			
			merger1.merge(merger2);						
			
			foundObjects.setElementAt(merger1,b-1);
	}
	
	/**
	 * Buffer was skipped.
	 *
	 * @param buffinfo buffer info to skip.
	 */
	protected void skip(BufferInfo buffinfo) {
		buffinfo.blip().x(-1);
		buffinfo.blip().y(-1);		
		
		foundObjects=new Vector();	
		
		buffinfo.setBlips(objectsFrame,foundObjects);
	}
	
	/**
	 * Get the last difference image. 
	 *
	 * @return the last difference frame.
	 */
	protected byte[] processed() {
		 return lastDifferenceFrame;
	}
	
	/** 
	 * Write the blip to disk.
	 * @param s stream to write to.
	 */
	private void writeObject(ObjectOutputStream s)
                                           throws IOException {
                      s.defaultWriteObject();
                      // customized serialization code
                      
                     	s.writeInt(threshold);
	 		s.writeByte(white);
			s.writeByte(black);
			s.writeInt(start);
        }
	
	/**
	 * Read the blip from the disk. 
	 * @param s stream to read from.
	 */
	private void readObject(ObjectInputStream s)
                                          throws IOException  {
                      try {s.defaultReadObject();

                      // customized deserialization code
		      	threshold=(int)s.readInt();
	 		white=(byte)s.readByte();
			black=(byte)s.readByte();
			start=(int)s.readInt();
                    
                      }
  		      catch (ClassNotFoundException e){System.err.println(e);}
        }
}
