package pinball.player.draw;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;


/**
 * Sets up a special filter to reverse the image, and find the hand in the frame.
 * The hand is the object furthest from the left part of the image. Since the camera
 * sees the person's right hand on the left side of the frame. The frame is reversed
 * so the person has an easier way of viewing themselves; like looking in a mirror.
 */
public class DrawStub extends Stub {
	/**
	 * The stub for the strategy.
	 *
	 * @param buffer The first buffer for the information buffers.
	 * @param gameStats the area on the screen to write stats info.
	 * @param strategy the strategy thread.
	 * @param camera the camera for the system.
	 */
	public DrawStub(Buffer buffer, TextArea gameStats, Strategy strategy, Camera camera) { 
		super(buffer,gameStats,strategy,camera);
	}
		
	/**
	 * Search for the flipper the number of times given. 
	 *
	 * @param quantity number of times to look for the flippers. 
	 */
	public void findFlippers(int quantity) {
	}
	
	/**
	 * Reverse the image from the camera. 
	 *
	 * @param pixels the pixels from the camera.
	 * @param w the width of the image.
	 * @param h the hight of the image.
	 */
	public void reverseImage(byte [] pixels, int w, int h) {
		byte swap;
					
		int height=0;
		for(int i=0; i<h; i++) {
			for(int j=0; j<w/2; j++) {
				try{
					swap=pixels[height+j];
					pixels[height+j]=pixels[height+w-j];
					pixels[height+w-j]=swap;
				}
				catch(Exception e){}
			}
			height+=w;
		}
		
	}
		
	
	/**
	 * Filter out the persons right hand.
	 * 
	 * @param findFlippers Is the system finding the flippers.
	 */
	public void filter(boolean findFlippers) {
		
		if(buffinfo.get()!=null) 
			reverseImage(buffinfo.get(), buffinfo.width(),buffinfo.height());
				
	
		/* The objects found. */
		Vector blips=getObjects();
		
		/* If no objects return. */
		if(blips.size()==0) {
			x=-1;
			y=-1;
			blipx(-1);
			blipy(-1);
			return;
		}
		
		/* The object. */
		Blip thing;
		
		/* The object with the min x. The further to the left on screen 
		the better chance it be the right hand. */
		Blip minX=(Blip)blips.elementAt(0);
		
		for (int i=1; i < blips.size(); i++) { 
			thing=(Blip)blips.elementAt(i);
			
			/* If it's futher to the right, it's more likely the right hand. */
			if(thing.xLocation()<minX.xLocation())
				minX=thing;
		}
		
		
		/* Set the hand. */
		thing=minX;
		x=thing.totalX() / thing.total();
		y=thing.totalY() / thing.total();
		blipx(x);
		blipy(y);
	}
	
	public void statistics() {}

	/**
	 * Allows the strategy to draw to the screen any lines or other graphics
	 * for the user to see how the strategy operates.
	 * This also draws the flipper's locations.
	 *
	 * @param g the graphics area to draw to.
	 * @param canvasWidth the width of the canvas.
	 * @param canvasHeight the height of the canvas.
	 */	
	public void strategyDraw(Graphics g, int wSize, int hSize) {		
		strategy.draw(g,wSize,hSize);
	}
}
