package pinball.player.handgrab;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;
import pinball.player.draw.*;

import java.awt.*;
import java.util.*;


/** The player draws objects on the screen for the person to touch. */
public class HandGrabStrategy extends Strategy {

	private int lastX=0, lastY=0;
	
	

	/** 
	 * The strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
  	 * @param camera the camera for the system.
	 */		
	public HandGrabStrategy(Buffer buffer, TextArea gameStats, Camera camera) {
		GameOptions.pixelThreshold(7);
		this.stub=new DrawStub(buffer,gameStats,this,camera);
		id="HandGrab";
	}
 				
				
	/**
	 * Should you flip or not. It never flips.
	 */
	public void decision() {
											
	}
	
	/**
	 * Draw some objects for the person touch. If they touch them, change the object's color.
	 *
	 * @param g the graphics to draw to.
	 * @param cavasWidth the width of the canvas.
	 * @param canvcasHeight the hight of the canvas.
	 */
	public void draw(Graphics g, int canvasWidth, int canvasHeight) {
		/* Size of box. */
		int boxW, boxH;
		
		/* Hands pointer. */
		Rectangle hand=null;
		
		boxW=canvasWidth/10;
		boxH=canvasHeight/3;
		
		Rectangle box1=new Rectangle(canvasWidth-canvasWidth/10,canvasHeight-canvasHeight/3,boxW,boxH);
		
		Rectangle box2=new Rectangle(canvasWidth-canvasWidth/10,0,boxW,boxH);
		
		g.setColor(Color.red);
		g.fill3DRect(box1.x,box1.y,box1.width,box1.height,true);

		g.setColor(Color.blue);
		g.fill3DRect(box2.x,box2.y,box2.width,box2.height,true);
			
		/* Hand is found. */
		if(stub.blipx()!=-1)			
			hand=new Rectangle(canvasWidth-stub.blipx(),stub.blipy(),canvasWidth/20,canvasWidth/20);
			
		/* Use last known location of hand. */
		if(stub.blipx()==-1)			
			hand=new Rectangle(canvasWidth-lastX,lastY,canvasWidth/20,canvasWidth/20);
		
		if(box1.intersects(hand)) {
		
			g.setColor(Color.green);
			g.fill3DRect(box1.x,box1.y,box1.width,box1.height,true);
		}
		
		if(box2.intersects(hand)) {
			g.setColor(Color.cyan); 				
			g.fill3DRect(box2.x,box2.y,box2.width,box2.height,true);					
		}

		/* Hand pointer. */
		g.setColor(Color.yellow);
		g.fill3DRect(hand.x,hand.y,hand.width,hand.height,true);
     
		if(stub.blipx()!=-1) {
			lastX=stub.blipx();
			lastY=stub.blipy();
		}
		

		/*
		Vector blips=stub.getObjects();
		
		System.err.println("Blips "+blips.size());
		*/
		
	}

}
