package pinball.player.handmouse;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

/**
 * Draws the frame from the camera on the screen. 
 */
public class HandMouseDrawingCanvas extends DrawingCanvas{
	/** The area in the image to look for the mouse. */
	private HandMouseZone zone;

	/** 
	 * Initalize the DrawingBoard. 
	 *
	 * @param w width of the image.
	 * @param h height of the image.
	 */
	public HandMouseDrawingCanvas(int w, int h){
		super(w,h);
		zone=new HandMouseZone(this);
		addMouseListener(zone);
		addMouseMotionListener(zone);			
	}


	/**
	 * Update the image's data. Draw to the off screen image.
	 *
	 * @param buffinfo the buffer information that has the image.
	 */
	public void updateFrame (BufferInfo buffinfo) {		
		/* The location of the ball. */
		int x=buffinfo.blip().x();
		int y=buffinfo.blip().y();
		 			  
		/* Make the image.*/
	        MemoryImageSource mis = 
        		new MemoryImageSource(buffinfo.width(),buffinfo.height(),
  		   	   		model, buffinfo.get(), 0, buffinfo.width());

	        /* The image from the camera. */
		Image myImage = createImage(mis);

		/* Draw the image of the game. */
		graphicsContext.drawImage(myImage,buffinfo.XOffset(),buffinfo.YOffset(),buffinfo.width(),buffinfo.height(),this);		

		zone.draw(graphicsContext);

		/* Draw the cross for the ball's location. */
		if(x>-1 && y>-1) {
			x+=buffinfo.XOffset();
			y+=buffinfo.YOffset();

			if(GameOptions.verbose())
				graphicsContext.setColor(Color.blue);
			else graphicsContext.setColor(Color.yellow);
			graphicsContext.drawLine(x,y-5,x,y+5);
			graphicsContext.drawLine(x+5,y,x-5,y);
		}

		paint(getGraphics());	
			
	}

}
