package pinball.player.handmouse;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;
import pinball.player.draw.*;

import java.awt.*;
import java.util.*;

/**
 * Find's the location of the blip in the image. 
 */
public class HandMouseProcessImage extends ProcessImage{
	protected int laser=63;
		
	/** 
	 * Make the first frame for comparison. 
	 *
	 * @param frame the frame used for comparison.
	 * @param x the width of the image.
	 */
	protected HandMouseProcessImage(byte[] frame, int x) {
		super(frame,x);
	}
	
	/**
	 * Analize the frame for the ball. 
	 *
	 * @param frame frame to analyze
	 * @param width the width of the image
	 * @param buffinfo the storage for the blips it finds.
	 */
	protected void process(byte[] frame, int x, BufferInfo buffinfo) {
		/* Object working on. */
		int currentObject=1;
		
		int xlocation,ylocation;
						
		blipLocationFrame=new byte[frame.length];

		objectsFrame=new int[x][frame.length/x];

		foundObjects=new Vector();		
		
		int maxX=x-1;
		int maxY=(frame.length/x)-1;

		boolean verbose=GameOptions.verbose();		
				
		int xat,yat;
				
		/* Frame dimentions. */
		xsize=x;
		ysize=frame.length/xsize;
		
		/* If the frame size has changed, make a new arrays. */
		if(lastFrameSize!=frame.length)
			setup(frame,x);
		
		/* Save the last frame's size for next compare. */
		lastFrameSize=frame.length;
		
		
									
		if(GameOptions.debug()>=1)
			System.err.println("Analyzing frame. Start="+start+" End="+frame.length);


		/* Amount pixel has changed. */
		int diff=0;		
		

		/* Reset pixel counter. */
		i=0;
		
		/* Compare the current image to the previous and look for changes.*/

		/* Look at all the pixels in the image. */
		for (ylocation=0; ylocation<= maxY; ylocation+=pixelRate) {			
			for (xlocation=0; xlocation<= maxX; xlocation+=pixelRate) {
				//System.err.println("pixel: "+i+" xlocation "+xlocation+" ylocation "+ylocation);
				
				
				if (frame[i]==laser) {
					currentDifferenceFrame[i] = black;
					blipLocationFrame[i] = black;
				}
				else {
					currentDifferenceFrame[i] = white;
					if(verbose)
						verboseFrame[i]=white;
					blipLocationFrame[i] = white;
				}													
	
				/* Exclusive OR the current difference frame and the blip location frame.
				   If there is a change note it. 
				   This is the new way the finds multiple objects.
				*/
				if (frame[i]==laser) {
					
					/* Make frame to be the blips image. */
					if(verbose)
						verboseFrame[i]=black;				

					/* Check to see if other pixels are from this blip. If so, merge or update the objects. */
				
					/* left pixel. */
					if((xlocation-pixelRate)>=0 && objectsFrame[xlocation-pixelRate][ylocation]>0) {
						/* If two connecting pixels are of two different objects merge them. */
						if(objectsFrame[xlocation][ylocation]>0 && objectsFrame[xlocation][ylocation]!=objectsFrame[xlocation-pixelRate][ylocation])
							merge(objectsFrame[xlocation][ylocation],objectsFrame[xlocation-pixelRate][ylocation]);
						objectsFrame[xlocation][ylocation]=objectsFrame[xlocation-pixelRate][ylocation];
					}
					
					/* upper left pixel. */
					if((xlocation-pixelRate)>=0 && (ylocation-pixelRate)>=0 && objectsFrame[xlocation-pixelRate][ylocation-pixelRate]>0) {
						if(objectsFrame[xlocation][ylocation]>0 && objectsFrame[xlocation][ylocation]!=objectsFrame[xlocation-pixelRate][ylocation-pixelRate])
							merge(objectsFrame[xlocation][ylocation],objectsFrame[xlocation][ylocation-pixelRate]);
						objectsFrame[xlocation][ylocation]=objectsFrame[xlocation-pixelRate][ylocation-pixelRate];
					}	
						
					/* upper pixel. */
 					if((ylocation-pixelRate)>=0 && objectsFrame[xlocation][ylocation-pixelRate]>0) {
						if(objectsFrame[xlocation][ylocation]>0 && objectsFrame[xlocation][ylocation]!=objectsFrame[xlocation][ylocation-pixelRate])
							merge(objectsFrame[xlocation][ylocation],objectsFrame[xlocation][ylocation-pixelRate]);
						objectsFrame[xlocation][ylocation]=objectsFrame[xlocation][ylocation-pixelRate];
						
					}
					
					/* upper right. */
					if((xlocation+pixelRate)<=maxX && (ylocation-pixelRate)>=0 && objectsFrame[xlocation+pixelRate][ylocation-pixelRate]>0) {
						if(objectsFrame[xlocation][ylocation]>0 && objectsFrame[xlocation][ylocation]!=objectsFrame[xlocation+pixelRate][ylocation-pixelRate])
								merge(objectsFrame[xlocation][ylocation],objectsFrame[xlocation+pixelRate][ylocation-pixelRate]);
						objectsFrame[xlocation][ylocation]=objectsFrame[xlocation+pixelRate][ylocation-pixelRate];
						
					}
			
					/* New blip found. */
 					if(objectsFrame[xlocation][ylocation]==0) {					
						foundObjects.addElement(new Blip());
						objectsFrame[xlocation][ylocation]=currentObject;
						thing=(Blip)foundObjects.elementAt(objectsFrame[xlocation][ylocation]-1);
						thing.alias(currentObject);				
						currentObject++;
						//System.err.println("An object is found.");
					}
					
	
	 				/* Increment the blip's pixel count. */
					thing=(Blip)foundObjects.elementAt(objectsFrame[xlocation][ylocation]-1);
					thing.xinc(xlocation);
					thing.yinc(ylocation);			
					
				}				
					
						
				/* Go to next pixel. */
				i++;
			}			
		}
		
		if(verbose)
			System.arraycopy(verboseFrame,0,frame,0,verboseFrame.length);

		maxObject=-1;
		
		thing=null;
 
		Vector foundObjects2=new Vector(); 
 		/* Filter out the merged objects. */
		for(int i=0; i<foundObjects.size();i++) {
			thing=(Blip)foundObjects.elementAt(i);
			if(thing.alias()-1==i) {
				foundObjects2.addElement(thing);			
			}
		}
 
		buffinfo.setBlips(objectsFrame,foundObjects2);
	
		lastXSize=xsize;
		lastYSize=ysize;	
		
		if(GameOptions.debug()>=1)
			System.err.println("Frame analyzed");
		
	}
	


}
