package pinball.player.handmouse;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

/**
 * Draws the frame from the camera on the screen. 
 */
public class HandMouseZone extends HandMouseMouseListeners{
	Point start, end, last;
	boolean gotIt=false; 
	HandMouseDrawingCanvas canvas;
	
	public HandMouseZone(HandMouseDrawingCanvas canvas) {
		this.canvas=canvas;		
		this.canvas.requestFocus();

		Rectangle box=Zone.get();
		start=new Point(box.x,box.y);
		end=new Point(box.width,box.height);		
	}

	public void draw(Graphics graphicsContext) {	
		if(start!=null) {
			graphicsContext.setColor(Color.red);
			if(end!=null)
				graphicsContext.drawRect(start.x,start.y,end.x,end.y);
			else if(end==null && last!=null)
				graphicsContext.drawRect(start.x,start.y,last.x,last.y);
		}
	}
	
	/** 
	 * Mouse is pressed add the location for the box. 
	 *
	 * @param event the mouse was pressed. 
	 */
	public void mousePressed(MouseEvent event) {
		if(!gotIt) {
			start=new Point(event.getX(),event.getY());
			last=new Point(0,0);
			end=null;
			gotIt=true;			
		}
		else if(gotIt) {
			end=new Point(event.getX(),event.getY());
			last=new Point(0,0);
			gotIt=false;
			Zone.set(new Rectangle(start.x,start.y,end.x,end.y));		
		}
		
		System.err.println("Pressed");
	}	
	/** 
	 * When the mouse is window, make sure the canvas has focus.
	 *
	 * @param event Mouse has entered the window.
	 */
	public void mouseEntered(MouseEvent event) {
		System.err.println("Entered");
		last=new Point(0,0);
		canvas.requestFocus();
	}  
	
	public void mouseMoved(MouseEvent event) {
		last=new Point(event.getX(),event.getY());
	}

	public void mouseDragged(MouseEvent event) {
		last=new Point(event.getX(),event.getY());
	}		
	
	/* Generic Mouse Events not used. */
	public void mouseClicked(MouseEvent event) {}
	/* Generic Mouse Events not used. */
	public void mouseReleased(MouseEvent event) {}
	/* Generic Mouse Events not used. */
	public void mouseExited(MouseEvent event) {}
	
	public Point start() {
		return start;
	}
	
	public Point end() {
		return end;	
	}
}
