package pinball.player.morsecode;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;

import java.awt.*;


/** Translate the flashes of morsecode to text.. */
public class MorseCodeStrategy extends Strategy {
	long lastFlash=System.currentTimeMillis();
	long lastOff=System.currentTimeMillis();
	
	/** Is the bulb on. */
	boolean bulb=false;
	
	String decode="";
	String string="";
	
	/** 
	 * The strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
  	 * @param camera the camera for the system.
	 */		
	public MorseCodeStrategy(Buffer buffer, TextArea gameStats, Camera camera) {
		this.stub=(Stub)new MorseCodeStub(buffer,gameStats,this,camera);
		id="mcode";
		MorseCodeStub morseStub=(MorseCodeStub)stub;
	}
 				
	/** For extended classes. */		
	public MorseCodeStrategy() {

	}

	public boolean timeRange(long target, long leeway, long time) {
		if(time>(target-leeway-1) && time<(target+leeway+1))
			return true;
		else return false;
	}
	
	/** For extended classes. */
	public void option(){
		/* Flash time. */
		if(buffinfo.blip().x()==-1 && bulbOn()) {
			long now=System.currentTimeMillis();		
			long elapsed=now-lastFlash;		
			lastOff=now;

			//System.err.println("Flash Time: "+elapsed);
			processFlash(elapsed);
			bulb=false;
		}

		/* Off time. */
		else if(buffinfo.blip().x()!=-1 && bulbOff()) {	
			long now=System.currentTimeMillis();		
			long elapsed=now-lastOff;		
			lastFlash=now;

			//System.err.println("Space Time: "+elapsed);
			processPause(elapsed);
			bulb=true;
		}
		
		/** Light is off and there is a string or decode. */
		else if(buffinfo.blip().x()==-1 && bulbOff() && (string.compareTo("")!=0 || decode.compareTo("")!=0)) {
			processPause(System.currentTimeMillis()-lastOff);
		}
	}	

	private void processFlash(long time) {
		/* DOT. */
		if(timeRange(MorseCode.dot,MorseCode.dot,time))
			setDot();
		/* DASH. */
		if(timeRange(MorseCode.dash,MorseCode.dot,time))
			setDash();
	}
				
	private void processPause(long time) {
		//System.err.println(time);
		/* END OF CHAR. */
		if(timeRange(MorseCode.charSpace,MorseCode.dot,time))
			endChar();
		/* END OF WORD. */
		if(timeRange(MorseCode.wordSpace,MorseCode.charSpace/2,time) || time>MorseCode.wordSpace) {
			endWord();
		}
	}

	private void setDot(){
		decode+=".";	
			
		display(".");		

	}
	
	private void setDash(){
 		decode+="-";		
		
		display("-");	
	}
	
	private void endChar(){
		char c=decodeChar(decode);
	
		/* Only encode valid symbols. */
		if(c!=0) {
			string+=c;
			display(" "+c+"\n");	
			decode="";
		}
	}
	
	private void endWord() {
		if(string!="") {
			display("\n"+string+"\n\n");			
			string="";
			decode="";
		}
	}	
	
	
	private void display(String text) {
		System.err.print(text);
	}
	
	/**
	 * Should you flip or not. It never flips.
	 */
	public void decision() {
											
	}
	
	private boolean bulbOn() {
		return bulb;
	}
	
	private boolean bulbOff() {
		return !bulb;
	}		
	
	public char decodeChar(String code){
		if(code.compareTo(".-")==0)
			return 'A';
		if(code.compareTo("-...")==0)
			return 'B';
		if(code.compareTo("-.-.")==0)
			return 'C';
		if(code.compareTo ("-..")==0)
			return 'D';
		if(code.compareTo(".")==0)
			return 'E';
		if(code.compareTo("..-.")==0)
			return 'F';
		if(code.compareTo("--.")==0)
			return 'G';
		if(code.compareTo("....")==0)
			return 'H';
		if(code.compareTo("..")==0)
			return 'I';
		if(code.compareTo(".---")==0)
			return 'J';
		if(code.compareTo("-.-")==0)
			return 'K';
		if(code.compareTo(".-..")==0)
			return 'L';
		if(code.compareTo("--")==0)
			return 'M';
		if(code.compareTo("-.")==0)
			return 'N';
		if(code.compareTo("---")==0)
			return 'O';
		if(code.compareTo(".--.")==0)
			return 'P';
		if(code.compareTo("--.-")==0)
			return 'Q';
		if(code.compareTo(".-.")==0)
			return 'R';
		if(code.compareTo("...")==0)
			return 'S';
		if(code.compareTo("-")==0)
			return 'T';
		if(code.compareTo("..-")==0)
			return 'U';
		if(code.compareTo("...-")==0)
			return 'V';
		if(code.compareTo(".--")==0)
			return 'W';
		if(code.compareTo("-..-")==0)
			return 'X';
		if(code.compareTo("-.--")==0)
			return 'Y';
		if(code.compareTo("--..")==0)
			return 'Z';
		if(code.compareTo("-----")==0)
			return '0';
		if(code.compareTo(".----")==0)
			return '1';
		if(code.compareTo("..---")==0)
			return '2';
		if(code.compareTo("...--")==0)
			return '3';
		if(code.compareTo("....-")==0)
			return '4';
		if(code.compareTo(".....")==0)
			return '5';
		if(code.compareTo("-....")==0)
			return '6';
		if(code.compareTo("--...")==0)
			return '7';
		if(code.compareTo("---..")==0)
			return '8';
		if(code.compareTo("----.")==0)
			return '9';
		return 0;
	}

}
