package pinball.player.movitrobot;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.part.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

/** Control the robot by keys as a test, but the robot can control itself. */
public class MovitRobotControls extends EventControls {

	/** States of the robot. */
	public static int FORWARD=1, BACKWARD=2, LEFT=3, RIGHT=4, STOPPED=0;

	/** Current system state. */
	private int state=STOPPED;
	
	private Panel panel;

	/** The last time a key was pressed. */	
	private long lastTime=System.currentTimeMillis();

	/** The controller for the robot. */
	public MovitRobotStub controller;
	
	/** Make the robot controls. */
	public MovitRobotControls() {
		System.err.println("CONTROLLER");
	}

	/** Set the controller of the robot. */
	public void setController(MovitRobotStub controller) {
		this.controller=controller;
	}

	/** 
	 * Mouse is pressed add the location of the mouse to the flipper zone. 
	 *
	 * @param event the mouse was pressed. 
	 */
	public void mousePressed(MouseEvent event) {
	}
	
	/* Generic Mouse Events not used. */
	public void mouseClicked(MouseEvent event) {}
	/* Generic Mouse Events not used. */
	public void mouseReleased(MouseEvent event) {}
	/* Generic Mouse Events not used. */
	public void mouseExited(MouseEvent event) {}

	/* Generic Keyboard Events. Ket the arrow keys to move the robot. */
	public void keyPressed(KeyEvent key) {
		if(GameOptions.running()!=true)
			return;
		if (key.getKeyCode() == KeyEvent.VK_UP) {
			if(state!=FORWARD){
				controller.forward();
				state=FORWARD;
			}
			else {
				controller.halt();
				state=STOPPED;
			}

		}

		if (key.getKeyCode() == KeyEvent.VK_DOWN) {
			if(state!=BACKWARD) {
				controller.backward();
				state=BACKWARD;
			}
			else {
				controller.halt();
				state=STOPPED;
			}

		}

		if (key.getKeyCode() == KeyEvent.VK_LEFT) {
			if(state!=LEFT){
				controller.counterClockwise();
				state=LEFT;
			}
			else {
				controller.halt();
				state=STOPPED;
			}

		}
	
		if (key.getKeyCode() == KeyEvent.VK_RIGHT) {
			if(state!=RIGHT) {
		 		controller.clockwise();
				state=RIGHT;
			}
			else {
				controller.halt();
				state=STOPPED;
			}
			
		}
		
		if (key.getKeyCode() == KeyEvent.VK_SPACE) {
			controller.halt();
			state=STOPPED;
		}				 						
	}
  
	/* Generic Keyboard. */
	public void keyReleased(KeyEvent key) {
	/*
	long time=System.currentTimeMillis();
	System.err.println(time-lastTime);
		controller.halt();
		state=STOPPED;
		lastTime=System.currentTimeMillis();
	*/
	}
	
	/** The last time the system stopped. */
	public long stoppedTime() {
		return lastTime;
	}
	
	/** The current status of the system. */
	public int status() {
		return state;
	}
	
	/** 
	 * When the mouse is window, make sure the canvas has focus.
	 *
	 * @param event Mouse has entered the window.
	 */
	public void mouseEntered(MouseEvent event) {
		panel.requestFocus();
	}
	
	/** The panel that contains the image from the camera. */
	public void setPanel(Panel p) {
		panel=p;
	}
	
	/** 
	 * Keys are pressed, check them. 
	 * 
	 * @param key the key pressed.
	 */
	public void keyTyped(KeyEvent key) {

	}
	
}
