package pinball.player.movitrobot;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.nohit.*;
import pinball.player.standard.*;
import pinball.player.network.*;
import pinball.player.remote.*;

import java.awt.*;

/**
 * Player which controls a robot. 
 */
public class MovitRobotPlayer extends StdPlayer{
	/** Id of class. */
	static { id="movit"; }
	
	/** 
	 * Start up the program. Takes command line arguments.
   	 * Takes the following command line arguments:
	 * -s for server mode.
	 * -c for client mode.	 
	 */
	public static void main(String args[]) {				
		StdPlayer player=new MovitRobotPlayer();
		player.makeIt(player,args);
	} 
	
	
	
	/** 
	 * Get the strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
  	 * @param camera the camera for the system.
 	 *
 	 * @return strategy thread.
	 */
	public Strategy localStrategy(Buffer buffer,TextArea gameStats, Camera camera) {
		return new MovitRobotStrategy(buffer,gameStats,camera);
	}
	
	
	/** 
	 * Get the strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
 	 *
 	 * @return strategy thread.
	 */
	public Strategy clientStrategy(Buffer buffer,TextArea gameStats) {
		return new NoHitRemoteStrategy(buffer,gameStats);
	}

	/** 
	 * Makes the player. 
	 *
	 * @param player Player to be.
	 * @param args Command line arguments.
	 */
	public void makeIt(StdPlayer player, String args[]) {
		/* Find out how the player is to be set up. */
		player.howPlay(args);
	
		/* Client mode. */
		if(player.how().equals(CLIENT)) {
			GameOptions.clientMode();		 	
			Strategy rp = player.clientStrategy(null,new TextArea());
		
			Thread remote=new Thread(rp);
		
			remote.start();			
		}
				
		
		/* Server mode. */
		else if(player.how().equals(SERVER)) {
			camera=new Camera(true);
			new StartSystem(args,player,"s");
		}
		
		/* Local mode. */
		else if(player.how().equals(LOCAL)) {
			camera=new Camera(true);
			new StartSystem(args,player,"l");
		}
		
		/* Incorrect command. Wrong strategy thread. */
		else {
			System.err.println("Error: Wrong Strategy Thread Command");
			System.exit(1);
		}
		
		System.err.println("Player: "+player.id());

	}
	

}
