package pinball.player.movitrobot;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;
import pinball.player.draw.*;

import java.awt.*;
import java.util.*;


/** Robot controller player. Can be controlled by the arrow keys. However,
currently it moves back and forth (reversing itself when it hits the wall. */
public class MovitRobotStrategy extends Strategy {	
	/** The menu for the system. */
	private RobotMenu menu;
	
	/** Used to get the mic sounds. */
	private SoundCard soundCard;
	
	/** The robot. */
	private MovitRobotStub robot;
	
	/** Used from the mic to change the state of the robot. */
	private int state=0;
	private int lastState=state;

	/** Ignore movement? */
	private boolean ignore=false;
	
	/** Previous X locations of the robot. */
	int lastX[]=new int[4];
	
	/** Last X location. */
	int lastLocation=0;
	
	/** Last time the robot moved. */
	long lastMoveTime;
	
	/** If the robot moving? */
	boolean moving=false;
	
	/** Time robot has to be moving, before it can be considered to be stopped. */
	long moveTimeThreshold=1000;

	/** Moved amount. */
	int lastDiffAvg=0;
	
	/** Number of times no movement. */
	int zeroMovement=0;
		
	/** 
	 * The strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
  	 * @param camera the camera for the system.
	 */		
	public MovitRobotStrategy(Buffer buffer, TextArea gameStats, Camera camera) {
		GameOptions.pixelThreshold(8);
		this.stub=new MovitRobotStub(buffer,gameStats,this,camera);
		menu=(RobotMenu)GameOptions.menu();
		menu.controls().setController((MovitRobotStub)this.stub);
		soundCard=new SoundCard();
		robot=(MovitRobotStub)this.stub;
		id="movit";		
	}
 				
	/** Do nothing for custom. */
	public void custom(){
	}	
	
	/** Find the robot stopped and reverse the direction. */
	public void option() {
	/*
		MovitRobotStub s=(MovitRobotStub)stub;
		long time=System.currentTimeMillis();		
		if(menu.controls().status()==MovitRobot.STOPPED) {
			if(time-menu.controls().stoppedTime()>50)
					s.halt();
		}
	*/		
		//test(soundCard.getSound());

		if(!moving) {
			moving=true;
			robot.forward();
		}
		
		int xTotal=0;
		int yTotal=0;
		
		int xTest=0;
		int yTest=0;
		
		/* The objects found. */
		Vector blips=buffinfo.getObjects();
		
		/* The object. */
		Blip thing;

		for (int i=0; i < blips.size(); i++) { 
			thing=(Blip)blips.elementAt(i);					
			
			x=thing.xLocation();
			y=thing.yLocation();								
			
			xTest=x+buffinfo.XOffset();
			yTest=y+buffinfo.YOffset();
			
			xTotal+=xTest;
			yTotal+=yTest;
		}
		
		/* Find the movement for the robot. */
		if(blips.size()>3) {
			int xAt=xTotal/blips.size();
			int yAt=yTotal/blips.size();
//			System.err.println("Robot is at "+xAt+","+yAt);
			lastX[lastLocation]=xAt;
			lastLocation++;
			if(lastLocation==lastX.length) {
				lastLocation=0;
				int min=lastX[0];
				int max=lastX[0];
				int diffTotal=0;
				
				/* Get the average movement. */
				for(int i=1; i<lastX.length; i++) {
					int value=lastX[i];
					if(value<min)
						min=value;
					if(value>max)
						max=value;
					diffTotal+=value;
				}
				int diff=max-min;
				int diffAvg=diffTotal/lastX.length;
 				//System.err.println("Diff: "+diff);
				//System.err.println("DiffAvg: "+diffAvg);
				
				/* Has the robot moved. */
				int diffDiff=Math.abs(diffAvg-lastDiffAvg);
				System.err.println("Diff: "+diffDiff);
			
				/* Find no movement. */
				if(diffDiff==0 && (System.currentTimeMillis()-lastMoveTime)>moveTimeThreshold)
					zeroMovement++;
				//else zeroMovement=0;
				
				/* If no movement, then switch position. */
				if(zeroMovement>=1) {
					switchState();
					zeroMovement=0;
				}
				lastDiffAvg=diffAvg;
			}

				
		}
	}

	/** Change the robot's state. */
	private void switchState() {
		if(robot.robotStatus()==MovitRobot.FORWARD){
			robot.backward();
		}
		else if(robot.robotStatus()==MovitRobot.BACKWARD)
			robot.forward();
		System.err.println("SWITCH");
		lastMoveTime=System.currentTimeMillis();
	}

	/** Test sound */
	public void test(byte[] sample) {
		byte voice[]=sample;
		int clapCount=0;
		int noise=0;
		
		int soundTotal=0;
		//System.err.println("SOUND: "+voice.length);
		for(int i=0;i<voice.length;i++) {
			noise=voice[i];
/*			if(noise>=0){
				noise=(127-noise);
			}
			if(noise<0)
				noise=(128+noise);
*/
			noise=Math.abs(noise);				
//			System.err.print(noise+" , ");
			if(noise>=40 && noise<127)
				soundTotal++;
		}
		//System.err.println("Sound Total "+soundTotal);

		if(ignore) {
			ignore=false;
			return;
		}		
		if(soundTotal>2)
			state++;
		if(state>5)
			state=0;
		if(state==0 && lastState!=state)
			robot.halt();
		if(state==1 && lastState!=state)
			robot.forward();
		if(state==2 && lastState!=state)
			robot.halt();
		if(state==3 && lastState!=state)
			robot.clockwise();
		if(state==4 && lastState!=state)
			robot.halt();
		if(state==5 && lastState!=state)
			robot.counterClockwise();
		if(lastState!=state)
			ignore=true;
		lastState=state;
//		System.err.println(state);		
	}


	/**
	 * No decision is called.
	 */
	public void decision() {
	}
	
	/**
	 * Draw nothing.
	 *
	 * @param g the graphics to draw to.
	 * @param cavasWidth the width of the canvas.
	 * @param canvcasHeight the hight of the canvas.
	 */
	public void draw(Graphics g, int canvasWidth, int canvasHeight) {

	}

}
