package pinball.player.movitrobot;

import pinball.player.standard.*;
import pinball.player.*;
import pinball.part.*;
import pinball.player.movitrobot.*;
import pinball.*;
import pinball.part.*;
import pinball.menuevent.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;

/** Control the robot by the arrow keys. */
public class RobotMenu extends MainMenu{
	/** Control the robot. */
	protected MovitRobotControls controls;

	/**
	 * Create the window and get the camera's settings
	 *
	 * @param Qcam The camera.
 	 * @param player The player of the game.
	 * @param stratThread Defines how the strategy is to player. Either local, client or server.
	 */
	public RobotMenu(Camera Qcam, StdPlayer player, String stratThread) {
		System.err.println("Robot Menu");
		controls=new MovitRobotControls();

		GameOptions.menu(this);

	        System.err.println("Starting");	   
	   
		this.Qcam=Qcam;
	   
		/* Get the image size. */
		width=Qcam.maxWidth*Qcam.zoomSize();
		height=Qcam.maxHeight*Qcam.zoomSize();
	
		System.err.println("Begining");  
	
		gameStats=new TextArea(5,30);

	    	/* Make the image buffers. */
		buffer=player.makeBuffers(GameOptions.bufferSize(),Qcam);
		
	    	/* Create the analyzing thread. */
		analysis=player.analyze(buffer);
	
	
		/* Create the Local strategy thread. */
		if(stratThread.equals("l"))
			strategy=player.localStrategy(buffer,gameStats,Qcam);
		/* Create the Server strategy thread. */			
		else if(stratThread.equals("s"))
			strategy=player.serverStrategy(buffer,gameStats,Qcam);
		/* Wrong strategy thread. */
		else {
			System.err.println("Error: Wrong Strategy Thread Command");
			System.exit(1);
		}

		/* Create the analyzing thread. */
		grabber=player.frameGrabber(Qcam,buffer);
		
		/* Create the post thread. */
		if (GameOptions.post()) {	
			post=player.post(buffer);
			if(GameOptions.display()) 
				picture=post.canvas(Qcam.xSize(),Qcam.ySize());
			
		}
		

		/* Make the window. */
	    	this.createWindow(width+boarder,height+boarder+50);


	}
	
	/** 
	 * Create the image window with canvas for image. 
	 *
	 * @param width The width of the window.
	 * @param height The height of the window.
	 */
	private void createWindow (int width, int height) {
		
	
		System.err.println("Going to create a window");
		this.setLayout(new GridLayout(3,1));
		
		/* The panels of the window. */
		this.add(top);
		this.add(middle);
		this.add(bottom);

		controls.setPanel(top);
		top.addKeyListener(controls);
		top.addMouseListener(controls);
		
		
		top.setLayout(new FlowLayout(FlowLayout.CENTER));
		middle.setLayout(new FlowLayout(FlowLayout.CENTER));
		bottom.setLayout(new GridLayout(2,1));
		
		bottom.add(systemButtons);
		bottom.add(gameButtons);				
		
		setSize(width,height);
		setTitle("QuickCam Image");
														
		/* Add the image canvas to the window. */
		if (GameOptions.display()) {
			top.add("Center",picture);
		}
		else top.add(new Label(""));
		

		/* Make the buttons. */
		exit=new Button();
		start=new Button();
		verbose=new Button();
		pause=new Button();
		
		exit.setLabel("Exit");
		start.setLabel("Start");
		verbose.setLabel("Difference");
		
		pause.setLabel("Pause System");
		
		exit.setBackground(Color.lightGray);
		start.setBackground(Color.lightGray);
		verbose.setBackground(Color.lightGray);
		pause.setBackground(Color.lightGray);

		middle.add(gameStats);

		
		/* Place the buttons in the window. */
				
		systemButtons.add(start);		
		systemButtons.add(exit);
		systemButtons.add(verbose);
		systemButtons.add(pause);
						
		System.err.println("Window made.");

	}	

	/** The controls to control the robot. 
	 *
	 * @return the robot controls.
	 */
	public MovitRobotControls controls() {
		return controls;
	}
	
	/** Startup the system. */
	public void begin() {
		PinballStart pinballStart;
		PinballBallLaunch launchBall;
	
		System.err.println("Adding events.");
	
		/* Button events. */
		exit.addActionListener(new PinballExit(Qcam));
		start.addActionListener(pinballStart=new PinballStart(this, post, grabber, analysis, strategy, start));
		verbose.addActionListener(new PinballVerbose(verbose));
		pause.addActionListener(new PinballPause(pause));

		System.err.println("Events added.");
		
		
		/* If not in batch mode, then place the window on the screen. */	
		if(!GameOptions.batchMode()) {
			
			setVisible(true);
			toFront();
			System.err.println("Window Done.");
		}
	}
	
}
