package pinball.player.multipleblip;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;

import java.awt.*;
import java.util.*;


/**  Player which blips if any blip is near the flippers.  */
public class MultipleBlipStrategy extends Strategy {

	private int blipDistanceMin=8;
	private int blipDistanceMax=14;

	/** 
	 * The strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
  	 * @param camera the camera for the system.
	 */		
	public MultipleBlipStrategy(Buffer buffer, TextArea gameStats, Camera camera) {

		this.stub=new Stub(buffer,gameStats,this,camera);
		id="multi";
		GameOptions.flipperAdjust(1.15);
	}
 				
	/** For extended classes. */		
	public MultipleBlipStrategy() {

	}
				
	/**
	 * Should you flip or not. It never flips.
	 */
	public void decision() {	
		/* Set the default that it did not flip. */	
		flipperFlipped(false);
		
		/* Is there a ball to flip? If so flip it.*/
		/* Add the offset value for the ball's location so it's
		correctly found from original location in game. */
		
		x=buffinfo.blip().x();
		y=buffinfo.blip().y();
		
//		System.err.println("Ball: "+x+","+y+" Flipper height "+flipperHeight);
//		System.err.println("LeftX: "+leftX+" RightX: "+rightX);
			
		if(x!=-1)	
			x+=buffinfo.XOffset();
		if(y!=-1)	
			y+=buffinfo.YOffset();
			

		Vector blips=buffinfo.getObjects();
		boolean nearLeft=false, nearRight=false;
		int distanceFromFlipper=0;

		for(int i=0; i<blips.size(); i++) {
			Blip thing=(Blip)blips.elementAt(i);
			boolean valid=false;
			int distanceFromLeft=-1;
			int distanceFromRight=-1;
			
			if(thing.yLocation()!=-1 && thing.width()<=GameOptions.ballMax() && thing.height()<=GameOptions.ballMax()) {
				valid=true;									
				x=thing.xLocation()+stub.XOffset();
				y=thing.yLocation()+stub.YOffset();			
			}

			if(!nearLeft && valid)
				distanceFromLeft=(int)Math.sqrt(Math.pow(leftX-x,2)+Math.pow(leftY-y,2));
			if(!nearRight && valid)
				distanceFromRight=(int)Math.sqrt(Math.pow(rightX-x,2)+Math.pow(rightY-y,2));
			
			if(distanceFromLeft>=blipDistanceMin && distanceFromLeft<=blipDistanceMax)
				nearLeft=true;
			if(distanceFromRight>=blipDistanceMin && distanceFromRight<=blipDistanceMax)
				nearRight=true;
			
			if(nearLeft && nearRight)
				break;

//			System.err.println(distanceFromLeft+" "+distanceFromRight);
		}
		/* Flip the flippers. */
		if(x>=0) {
		
				/* Flip left flipper. */
				if(nearLeft && stub.isLeftDown()) {
					stub.leftFlipperState(true);
				 }
				/* Flip right flipper. */
				if(nearRight && stub.isRightDown()) {
					stub.rightFlipperState(true);
				 }
		}
	
											
	}
	
	
	/**
	 * Allows the strategy to draw to the screen any lines or other graphics
	 * for the user to see how the strategy operates.
	 *
	 * @param g the graphics area to draw to.
	 * @param canvasWidth the width of the canvas.
	 * @param canvasHeight the height of the canvas.
	 */
	public void draw(Graphics g, int canvasWidth, int canvasHeight) {
		g.setColor(Color.green);
		circle(g,leftX,leftY,blipDistanceMax);
		g.setColor(Color.red);
		circle(g,leftX,leftY,blipDistanceMin);

		g.setColor(Color.green);
		circle(g,rightX,rightY,blipDistanceMax);
		g.setColor(Color.red);
		circle(g,rightX,rightY,blipDistanceMin);		
		
	}

	/**
	 * Draws a circle on the screen.
	 *
	 * @param g the graphics to draw to.
	 * @param x the x location.
	 * @param y the y location.
	 */
	private void circle(Graphics g, int xStart, int yStart, int size) {
		int halfSize=size/2;
		g.fillOval(xStart-halfSize,yStart-halfSize,size,size);
	}

}
