package pinball.player.nopinball;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;


/**
 * The hand is the object furthest from the left part of the image. Since the camera
 * sees the person's right hand on the left side of the frame.
 */
public class NoPinballStub extends Stub {
	/**
	 * The stub for the strategy.
	 *
	 * @param buffer The first buffer for the information buffers.
	 * @param gameStats the area on the screen to write stats info.
	 * @param strategy the strategy thread.
	 * @param camera the camera for the system.
	 */
	public NoPinballStub(Buffer buffer, TextArea gameStats, Strategy strategy, Camera camera) { 
		setupImage();
		this.camera=camera;
		stats=new Stats(this);
		
		this.buffer=buffer;
		this.gameStats=gameStats;
		this.strategy=strategy;
		buffinfo=buffer.info();
		
		strategy.setQueue(queue);
		System.err.println("* No Pinball Stub");
	}
		

	/** 
	 * Filter out the object.
	 */
	public void run() {		
		/* Get the lock. */
		getLock();
		
		/* Initalize the strategy. */
		strategy.initalize();
		
		/* Setup the stub for the system. */
		buffinfo.setStub(this);
		
		/* Release the lock. */
		releaseLock();
		
		/* Next buffer. */
		nextBuffer();	

		/* Check the frame to see if flipping is needed. */
		while(true) {

			
			/* Get the lock and get the info. */
			getLock();
				
			/* If not skipping the buffer, process it. */
			if(!buffer.skipped())	{
				/* If the system is not paused, figure out what to do based on the blips found in the frame. */
				if(!GameOptions.pause()) {	
					
					process();
					
				}
				else if(GameOptions.pause()) {
					skipBuffer();
				}
		       							 
			}
			
			/* Skipped frame. */
			else if(buffer.skipped()) {
				skipBuffer();
			}
			
	       		/* Go to the next buffer. */
			nextBuffer();
			
			/* Skip frames. */
			//skip();
				
			/* Let another thread go. */
			Thread.yield();
		}
	}


	/**
	 * Search for the flipper the number of times given. 
	 *
	 * @param quantity number of times to look for the flippers. 
	 */
	public void findFlippers(int quantity) {
	}
		
	
	/**
	 * Filter out the persons right hand.
	 * 
	 * @param findFlippers Is the system finding the flippers.
	 */
	public void filter(boolean findFlippers) {
		/* The objects found. */
		Vector blips=getObjects();
		
		/* If no objects return. */
		if(blips.size()==0) {
			x=-1;
			y=-1;
			blipx(-1);
			blipy(-1);
			return;
		}
		
		/* The object. */
		Blip thing;
		
		/* The object with the min x. The further to the left on screen 
		the better chance it be the right hand. */
		Blip minX=(Blip)blips.elementAt(0);
		
		for (int i=1; i < blips.size(); i++) { 
			thing=(Blip)blips.elementAt(i);
			
			/* If it's futher to the left of the image, it's more likely the right hand. */
			if(thing.xLocation()<minX.xLocation())
				minX=thing;
		}
		
		
		/* Set the hand. */
		thing=minX;
		x=thing.totalX() / thing.total();
		y=thing.totalY() / thing.total();
		blipx(x);
		blipy(y);
	}
	
	public void statistics() {}
	
	/**
	 * Allows the strategy to draw to the screen any lines or other graphics
	 * for the user to see how the strategy operates.
	 * This also draws the flipper's locations.
	 *
	 * @param g the graphics area to draw to.
	 * @param canvasWidth the width of the canvas.
	 * @param canvasHeight the height of the canvas.
	 */	
	public void strategyDraw(Graphics g, int wSize, int hSize) {}
}
