package pinball.player.random2;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;
import pinball.player.remote.*;

import java.awt.*;

/** Player who randomly flips the flippers. */
public class Random2Strategy extends Strategy {
	/** Time to rest between flips. */
	private int rest=GameOptions.resetTime()*2;	
	
	/** Did flip take place. */
	private boolean flip=false;
	
	/** Last time flipped. */
	private long lastTime;

	/** 
	 * The strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
  	 * @param camera the camera for the system.
	 */	
	public Random2Strategy(Buffer buffer, TextArea gameStats, Camera camera) {
		this.stub=new Stub(buffer,gameStats,this,camera);
		
		id="Random2";
		boarder=16;
		stub.boarder(boarder);	
		lastTime=System.currentTimeMillis();
	}	
	
	/**
	 * Required for extended class.
	 */
	public Random2Strategy(){}
	
	/** 
 	 * Should you flip or not. 
 	 * 1/3 flip left, 1/3 right & 1/3 both.
	 */
	public void decision() {			
		if(!stub.statsBallInPlay() || (System.currentTimeMillis()-lastTime)<rest)
			return;

		/* Random choice for rest. */
		rest=(int)(Math.random()*GameOptions.resetTime()*2);	

		/* Reset flippers. */
		if(flip==false) {
				stub.leftFlipperState(false);
				stub.rightFlipperState(false);
				flip=true;
		}
	
		/* Flip flipper(s). */
		else if(flip==true) {				
				stub.leftFlipperState(true);
				stub.rightFlipperState(true);
				flip=false;				
		}

		lastTime=System.currentTimeMillis();
	}
}
