package pinball.player.standardV2;

import pinball.*;
import pinball.buffer.*;
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

/**
 * Gather the system inputs, currently takes images from the camera.
 */
public class Input extends FrameGrabber {
	protected StandardInputs inputDevices;
		
	/** The current frame number. */
	protected int frameNumber=1;
	
	/** The current infomation buffer. */
	protected Buffer buffer;
	
	/** The contents of the buffer. */
	protected BufferInfo buffinfo;
	
	/** Did the resize of the camera take place. */
	protected boolean resized=false;
	
	/** Amount of buffers adjusted. */
	protected int edited=0;
	
	/** Image location. */
	protected int X,Y,XOffset,YOffset;
	
	/** Is the image to be resized. */
	protected boolean resizing=false;
		

	/**
	 * Make a new frame grabber thread.
	 *
	 * @param buffer first buffer in the series of buffers.
	 * @param devices the devices for the system.
	 */
	public Input(Buffer buffer, StandardInputs devices) {
 		/* The first buffer to store the information in. */
		this.buffer=buffer;
		
		/* Get the devices for the system. */
		this.inputDevices=devices;
	}
	
	/**
	 * For extended classes.
	 */
	public Input() {}
	
	/**
	 * Get the name of the object.
	 * 
	 * @param name of the object.
	 */
	public String toString() {
		return "Input";
	}
	
	/**
	 * Grab images from the camera. 
	 */
	public void run() {
		
				
		buffinfo=buffer.info();

		/* First frame grabbed. */
		buffer.lock().grab(buffer.id());
		buffinfo.time().setFirstGrab();
		buffinfo.time().setStartGrabTime();

		/***********************************/	
		/* Take a picture from the camera. */
		/***********************************/	
		
		/* Wait until first frame is grabbed. */
		CameraFrame cameraFrame=null;
		while(cameraFrame==null){
			cameraFrame=inputDevices.camera().getFrame();
			System.err.print(".");
			Thread.yield();			
		}
  		buffinfo.set(cameraFrame);
		PinballLog.writeln(frameNumber,"FRAME_GRABBED",1,this);
		

		if(GameOptions.debug()>=2)
	        	System.err.println("Captured frame #"+frameNumber);    		
	     
		buffinfo.number(frameNumber);
    	          buffinfo.time().setEndGrabTime();
    	        
		frameNumber++;
		
       		buffer.lock().grabed(buffer.id());				        	
       		
	
		buffer=buffer.next();
	
		
		Thread.yield();
		
		/***********************************/
		/* The rest of the frames grabbed. */
		/***********************************/

		while(true) {		
		
			buffer.lock().grab(buffer.id());
			
			PinballLog.writeln(frameNumber,"FRAME_GRABBED",1,this);

			buffinfo=buffer.info();
			
		
			buffinfo.time().setStartGrabTime();
		
			/*************************/
			/*    SET UP RESIZE. 	*/
			/***********************/
			/* If the buffers are to be resized. Go through each of the buffers and resize them. */
			if(buffinfo.resize() || resizing) {
	
				if(edited<=GameOptions.bufferSize()) {			
					resizeImage();
				}
				else resizing=false;
			}							
					
			/* Check if frame was ready. */ 
			cameraFrame=null;	
			cameraFrame=inputDevices.camera().getFrame();

			if(cameraFrame!=null) {
				buffinfo.set(cameraFrame);
				buffer.skipped(false);
			}
			else	{
				//System.err.println("Missing frame");
				buffer.skipped(true);
			}

			if(GameOptions.debug()>=2)
	        		System.err.println("Captured frame #"+frameNumber);    		
	     
			buffinfo.number(frameNumber);
    		        buffinfo.time().setEndGrabTime();
    	        	


			frameNumber++;		
		
	       		buffer.lock().grabed(buffer.id());				        		
       				
			buffer=buffer.next();
					

			Thread.yield();
       		}    	
       	}	
	
	/** 
	 * Check to see if resize of the images is needed.
	 */
	protected void resizeImage() {
		Camera Qcam=null;
			/* Resize the images, if not already done. */
			if(!resized) {
				resize();								
				resized=true;
			}
			
			/* get the new dimensions.*/
			buffinfo.width(Qcam.xSize());
			buffinfo.height(Qcam.ySize());
			buffinfo.offsets(Qcam.XOffset(),Qcam.YOffset());
			buffinfo.cameraInfo(Qcam.xSize()*Qcam.ySize());
			
			/* Set the resize flag to resized. */
			buffinfo.resized();	
			
			/* Buffer needs to clear the display when drawn. */					
			buffinfo.clearing(true);
			
			/* Increment the amount of buffered adjusted.*/
			edited++;
			
			/* If done with all the buffers. */
			if(edited>=GameOptions.bufferSize()) {
				edited=0;
				resized=false;
				resizing=false;
				PinballLog.writeln(buffinfo.number(),"imageX="+Qcam.xSize()+" imageY="+Qcam.ySize(),1,this);
			}			
	}
	
	/**
	 * Setup the resize. 
	 */
	protected void resize() {			 
		Camera Qcam=null;
		if(buffinfo.resize()==true) {
			System.err.println("Resize started with frame: "+buffinfo.number());
			X=buffinfo.resizeX();
			Y=buffinfo.resizeY();
			XOffset=buffinfo.resizeXOffset();
			YOffset=buffinfo.resizeYOffset();
		}
	
		/* Resize this frame. */
		Qcam.resizeFrame(buffinfo);
		resizing=true;
					
	}
}
