package pinball.player.standardV2;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

/**
 * The image grabber. Takes images from the camera.
 */
public class PictureTaker extends Thread {
	/** The camera. */
	protected Camera camera;

	/** The frame from the camera. */
	protected  CameraFrame frame=null;
	
	/* The original style information of the buffer. 
	For backward compatibility.*/

	/** Used for live captured. */
	private BufferInfo liveInfo;
			
	/** Used to capture the info. */
	private BufferInfo info[]=new BufferInfo[2];
	
	/** Is the info locked. */
	private boolean locked=false;
	
	/** Used for time counting. */
	private long startTime=System.currentTimeMillis();
	private long lastTime=System.currentTimeMillis();

	/** Frames captured. */
	private int frames=0;

	/** If you want to limit the time between frame captures. */
	private long frameRate=50;

	/** Used to keep track of time. */
	private long lastPicture=System.currentTimeMillis();
	private long lastRequest=System.currentTimeMillis();
	
	/** Time counters. */
	private long grabTime=50, requestTime=0, padTime=5;
	
	/** 
	 * Take pictures on that camera. 
	 * @param camera camera to take pictures.	
	 */
	public PictureTaker(Camera camera) {
		this.camera=camera;
		
		/* Image size, brightness, and contrast. */
		PinballLog.headerInfo("imageX="+camera.xSize()+" imageY="+camera.ySize(),this);
		PinballLog.headerInfo("brightness="+camera.brightness(),this);
		PinballLog.headerInfo("contrast="+camera.contrast(),this);
		
		//info[0]=new Buffer(1).info();
		//info[1]=new Buffer(2).info();
		
		liveInfo=new Buffer(1).info();
		liveInfo.cameraInfo(camera.frameSize(),camera.xSize(),camera.ySize());
	}

	/**
	 * Grab images from the camera. 
	 */
	public void run() {
		setPriority(Thread.NORM_PRIORITY+1);

		float FPS=0;
		startTime=System.currentTimeMillis();
		while(true){
		
			long elapsed=System.currentTimeMillis()-lastPicture;
			//System.err.println("Picture Time: "+elapsed);
		
			/*************************/
			/*    LOG CHANGE 	*/
			/***********************/
			if(GameOptions.logChange()) {
				PinballLog.newBall();
				camera.nextBall();
				GameOptions.logChange(false);
			}	    	
			//if(System.currentTimeMillis()-lastTime>=frameRate) {
				long start=System.currentTimeMillis();

				camera.grabFrame(liveInfo);

				elapsed=System.currentTimeMillis()-start;
				//System.err.println("Grab Time: "+elapsed);
				grabTime=elapsed;

				liveInfo.getCameraFrame().setID(frames);
				frames++;
				long time=System.currentTimeMillis()-startTime;
				time/=1000;
				if(frames%50==0){
					FPS=frames;
					FPS/=time;
					System.err.println("FPS: "+FPS);
				}
				switchFrame();
				//System.err.print("*");
				lastTime=System.currentTimeMillis();
			//}

			lastPicture=System.currentTimeMillis();

			try{		
				long sleepTime=requestTime+elapsed;
				//System.err.println("Sleep time: "+sleepTime);

				/* At least 5 ms delay. */
				requestTime=5;
				sleep(sleepTime);
			}
			catch(Exception e){System.err.println(e);}
		}
	}

	/** Go to next buffer. */
	private synchronized void switchFrame() {
		frame=liveInfo.getCameraFrame();
		
		/* Get a new frame to store the next image. */
		liveInfo.cameraInfo(camera.frameSize(),camera.xSize(),camera.ySize());

		/* If there is nothing locked, then lock this buffer
		and go to the next buffer in the series. */

	}

	/**
	 * Get the frame from the camera. 
	 *
	 * @returns frame from camera.
	 */
	public synchronized CameraFrame getFrame() {
		long elapsed=System.currentTimeMillis()-lastRequest;
		//System.err.println("Request Time: "+elapsed);

		CameraFrame photo=frame;		
		if(frame!=null) {
			//System.err.println("ID: "+frame.getID()+" TIME: "+System.currentTimeMillis());
			frame=null;
			requestTime=elapsed;
		}
		else requestTime=0;
		
		lastRequest=System.currentTimeMillis();
		return photo;
	}
}
