package pinball.player.standardV2;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;


/**
 * Controller for the standardplayerv2.
 */
public class StandardV2Stub extends Stub {
	
	/**
	 * The stub for the strategy.
	 *
	 * @param buffer The first buffer for the information buffers.
	 * @param gameStats the area on the screen to write stats info.
	 * @param strategy the strategy thread.
	 * @param camera the camera for the system.
	 */
	public StandardV2Stub(Buffer buffer, TextArea gameStats, Strategy strategy, Camera camera) { 
		super(buffer,gameStats,strategy,camera);
	}

	/**
	 * Search for the flipper the number of times given. 
	 *
	 * @param quantity number of times to look for the flippers. 
	 */
	public void findFlippers(int quantity) {
		int ignoreAmount=3;
		
		/* Current flipper test number. 
		   Ignore the first few times the flippers are checked. 
		*/
		int at=-ignoreAmount; 
		
		/* X flipper test values. */	
		int []x=new int[quantity];
		
		/* Y flipper test values. */
		int []y=new int[quantity];
		
		/* Still testing flippers? */
		boolean findingFlippers=true;
		
		/* Used to sum up the values. */
		int total=0;
		
		/* Which flipper is being tested. */
		int testing=LEFT;
		
		/* Center of the change of the image. Used to find the center of the flipper. */
		Location center;
		
		System.err.println("Testing Flippers.");
		startTime=System.currentTimeMillis();
		
		/* Find the flipeprs. */
		while(findingFlippers) {
			getLock();
			
			while(buffer.skipped()) {
				time().reset();
				releaseLock();											
				nextBuffer();
				getLock();
			}				
			
			/* Write the blips found to the log file. */
			storeBlips();
	
			filter(true);	
	
			/* See if it's time to reset the left flipper. */	
			if(leftFlipperReset(pause) && isLeftUp()) {
				leftFlipperState(false);
				startTime=System.currentTimeMillis();
			}
			
			/* See if it's time to reset the right flipper. */				
			else if(rightFlipperReset(pause) && isRightUp()) {
				rightFlipperState(false);
				startTime=System.currentTimeMillis();	
			}

			/* Test the right flipper. */
			else if(testing==RIGHT && isRightDown() &&
			System.currentTimeMillis()-startTime>=pause) {
				rightFlipperState(true);
				startTime=System.currentTimeMillis();

/*								
				try {
        		       		Thread.sleep(1000);
	      			} 
		       		catch (InterruptedException e)
		       		{System.err.println(e);}
*/
				
			}
			/* Test the left flipper. */
			else if(testing==LEFT  && isLeftDown() &&
			System.currentTimeMillis()-startTime>=pause) {				
				leftFlipperState(true);
				startTime=System.currentTimeMillis();
/*				
				try {
        		       		Thread.sleep(1000);
	      			}  
		       		catch (InterruptedException e)
		       		{System.err.println(e);}
*/
			
			}

			center=getCenterChange();
			System.err.println("Buffer: "+getBuffNumber()+" "+center.x()+","+center.y());

			/* Store flipper locations. -1 means not found. */
			if(center.x()!=-1 && center.y()!=-1) {
				if(at>-1) {
					/* Adjust the center by some percent lower on the screen. */
					center.y((int)(center.y()*GameOptions.flipperAdjust()));
					x[at]=center.x();
					y[at]=center.y();
					blipx(center.x());
					blipy(center.y());
//					System.err.println("Movement "+x[at]+","+y[at]);
					PinballLog.writeln(number(),"FLIPPER FlipperX="+x[at]+" FlipperY="+y[at],1,this);
				}				
				at++;
			}

			/* Done with the left flipper. */			
			if(at==quantity && testing==LEFT) {
			
				/* Set up to test right flipper. */				
				testing=RIGHT;
				at=-2;
				total=0;
				
				leftFlipperState(false);
				
				/* Average the x. */
				for(int i=0;i<quantity;i++)
					total+=x[i];
				left.x(total/quantity);
				
				total=0;
				
				/* Average the y. */
				for(int i=0;i<quantity;i++)
					total+=y[i];
				left.y(total/quantity);
				
				System.err.println("Left flipper at: "+left.x()+","+left.y());				
				
				left.x(left.x());
				left.y(left.y());

				leftX=left.x();
				leftY=left.y();
	
/*				
 				try {
        		       		Thread.sleep(1000);
	      			} 
		       		catch (InterruptedException e)
		       		{System.err.println(e);}
*/
				
				/*Keep waiting until the left flipper stopped moving. */
				while(center.x()!=-1 && center.y()!=-1) {
					System.err.println("Ignored "+getBuffNumber()+" "+center.x()+","+center.y());
					releaseLock();
					nextBuffer();
					getLock();
					center=getCenterChange();
					time().reset();																													
					Thread.yield();
					
/*					
					try {
         			       		Thread.sleep(1000);
		      			}
					catch (InterruptedException e)
			       		{System.err.println(e);}
*/

				}
			
			}
			
			
			/* Done with the right flipper. */
			else if(at==quantity && testing==RIGHT) {
				findingFlippers=false;
				
				total=0;
				
				/* Average the x. */
				for(int i=0;i<quantity;i++)
					total+=x[i];
				right.x(total/quantity);
				
				total=0;
	
				/* Error occured. Right x is less than Left x.
				*/			
				if(right.x()<left.x()) {
					System.err.println("Refinding flippers.");
					PinballLog.headerInfo("flipperError=Refinding_Flippers",this);
					testing=LEFT;
					at=-ignoreAmount;
					total=0;
					findingFlippers=true;
				}
					
				
				/* Average the y. */
				for(int i=0;i<quantity;i++)
					total+=y[i];
				right.y(total/quantity);
				System.err.println("Right flipper at: "+right.x()+","+right.y());
				
				rightFlipperState(false);

				right.x(right.x());
				right.y(right.y());
								
				if(right.y()<left.y())
					flipperOffset(right.y());
				else flipperOffset(left.y());
				
				rightX=right.x();
				rightY=right.y();
				
			}
			
			
			time().reset();
			releaseLock();
											
			nextBuffer();
		
			yield();
		}

		/* Find the higher flipper and then set the hight to start to draw
		   the flipper zone to test for flipper hits. 
		*/
		if(leftY>rightY) {
			//flipperZoneSize=leftY-GameOptions.flipperThreshold();
			flipperZoneSize=GameOptions.flipperZoneSize();
			outZone=rightY+fromBottom;
		}
		else {
			//flipperZoneSize=rightY-GameOptions.flipperThreshold();
			flipperZoneSize=GameOptions.flipperZoneSize();
			outZone=leftY+fromBottom;
		}
		
		flipperZone=0;
		
		outZoneEnd=outZone+outZoneLength;
	
		/* The flipper zone is in seen. Refind the flippers. */	
		if((flipperZone+flipperZoneSize)<1) {
			System.err.println("Flippers too high to the top.");		
			PinballLog.headerInfo("flipperError=Refinding_Flippers",this);
			GameOptions.refindFlippers(true);
		}
		
		/* Set the flipper zone area. */
		GameOptions.flipperZone(flipperZone);

		System.err.println("Done testing.");
		
	}	

	
}
