package pinball.player.tracker;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;

import java.awt.*;
import java.util.*;
import java.lang.Math;

/** The player tracks the ball's progress. */
public class TrackerStrategy extends Strategy {
	/** Previous ball location's. */
	protected Vector locations=new Vector();

	/** Number of previous locations found. */
	protected int maxLocations=6;

	/** Last ball's location. */
	protected int lastX=0,lastY=0;
	
	/** Time between missed ball's till previous locations are erased. */
	protected long lastTime=System.currentTimeMillis();

	/** 
	 * The strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
  	 * @param camera the camera for the system.
	 */		
	public TrackerStrategy(Buffer buffer, TextArea gameStats, Camera camera) {
		GameOptions.pixelThreshold(8);
		this.stub=new Stub(buffer,gameStats,this,camera);

		boarder=3;
		stub.setPriority(Thread.NORM_PRIORITY);
		stub.boarder(boarder);
		GameOptions.ballMin(5);
		GameOptions.ballMax(15);
		GameOptions.pixelThreshold(10);
		GameOptions.moveThreshold(5);
	}
 				
				
	/**
	 * Find the angle and store previous locations.
	 */
	public void special() {
		/* Ball not found. */
		if(x==-1 && y==-1) {
			long now=System.currentTimeMillis();
			/* The ball went out of the screen, so remove the
			previous location listing. 
			*/
			if((now-lastTime)>1000) {				
				locations.removeAllElements();
			}						
			return;
		}
		lastTime=System.currentTimeMillis();
		if(locations.size()==maxLocations)
			locations.removeElementAt(0);
		locations.addElement(new Point(x,y));		
		
		int deltaX=x-lastX;
		int deltaY=y-lastY;
		
		lastX=x;
		lastY=y;
		
		if(deltaX==0) {
			System.err.println("no slope");
			return;
		}
		
		double slope=deltaY;
		slope /=deltaX;
		
		//System.err.println("Slope: "+slope+" Changes:"+deltaX+" "+deltaY);
		double angleRadians=Math.atan(slope);
		double angle=(angleRadians*180)/Math.PI;
		System.err.println("Angle: "+angle);
	}
	
	/**
	 * Draw some objects for the person touch. If they touch them, change the object's color.
	 *
	 * @param g the graphics to draw to.
	 * @param cavasWidth the width of the canvas.
	 * @param canvcasHeight the hight of the canvas.
	 */
	public void draw(Graphics g, int canvasWidth, int canvasHeight) {
		Point location, location2;
		g.setColor(Color.yellow);
		
		int space=4;
		int halfSpace=space/2;
		
		//System.err.println("BALLS");
		
		for(int i=0;i<locations.size();i++) {
			location=(Point)locations.elementAt(i);
			if(i<(locations.size()-1))
				location2=(Point)locations.elementAt(i+1);
			else location2=(Point)locations.elementAt(i);
			//g.fillOval(location.x-halfSpace,location.y-halfSpace,space,space);
			g.drawLine(location.x,location.y,location2.x,location2.y);
			//System.err.println(locations.size()+" "+location.x+" "+location.y+" : "+location2.x+" "+location2.y);
		}	
	}
}
