package pinball.server;

import pinball.*;
import pinball.part.*;

import java.net.*;
import java.io.*;

/**
 * The server part of the network.
 */
public class MyServer extends Network{

	/* Network socket. */
        private ServerSocket 	sct;      
	
	/** Data port. */  
        private int 		port;
	
	/** Data socket. */
        private Socket 		socket;	
	
	/** Is the system online? */
	private boolean online=false;

	/** Going to go online. */
	protected boolean goingOnline=true;	
	
        /** Timeout. */
	private int timeout=45000;

        /**
         * Set up the sockets for the server. 
         */		
	public MyServer() {
		port=GameOptions.networkPort();
		try { sct = new ServerSocket(port);
		} catch (Exception e) {System.err.println(e);}
		
		connection();
	}
	

        /**
         * Set up the sockets for the server.
	 * @param port the port to connect on.
         */		
	public MyServer(int port) {
		this.port=port;
		try { sct = new ServerSocket(port);
		} catch (Exception e) {System.err.println(e);}
		
		connection();
	}

        /**
         * Set up the sockets for the server.
	 * @param port the port to connect on.
	 * @param timeout the timeout for the port.
         */		
	public MyServer(int port,int timeout) {
		this.port=port;
		this.timeout=timeout;
		try { sct = new ServerSocket(port);
		} catch (Exception e) {System.err.println(e);}
		
		connection();
	}
	
	/**
         * Set up the sockets for the server.
	 * @param port the port to connect on.
	 * @param timeout the timeout for the port.
	 * @param connect should the server wait for a connection now.
         */		
	public MyServer(int port,int timeout, boolean connect) {
		this.port=port;
		if(timeout!=0)
			this.timeout=timeout;
		try { sct = new ServerSocket(port);
		} catch (Exception e) {System.err.println(e);}
		
		if(connect)
			connection();
	}
		
	/** 
	 * Wait for a connection 
	 */
	public void connection() {
		/* Open the socket. */
		try {
			System.err.println("Waiting for connection.");
			Socket socket = sct.accept();

			socket.setSoTimeout(timeout);

 			in = new BufferedReader (
			     new InputStreamReader(socket.getInputStream()));
		        out = new PrintWriter (
			     socket.getOutputStream(),false);			
		} catch (Exception e) {System.err.println(e);System.exit(1);}
		
		online=true;
		System.err.println("Server is online.");
		//new StartButton(GameOptions.getDataIO()).press();
		GameOptions.launchBall(true);
		GameOptions.pause(false);
	}	
	 
	/**
         * Setup the client with pre-exisiting streams.
         */
	public MyServer(BufferedReader 	in, PrintWriter out) {
		this.in=in;
		this.out=out;
	}	
	
	/**
	 * Set server offline. 
	 */	
	public void offline() {
	 	online=false;
		goingOnline(true);
		GameOptions.pause(true);
	}
	
	/**
	 * Is the server online. 
	 * @return Online status.
	 */ 
	public boolean isOnline() {
		return online;
	}

	/**
	 * Get the going online status. 
	 *
	 * @return going online status.
	 */ 
	public boolean goingOnline() {
		return goingOnline;
	}
	
	/**
	 * Set the going online status.
	 *
	 * @param online set the online status.
	 */
	public void goingOnline(boolean online) {
		goingOnline=online;
	}
	
	/**
	 * Get the server socket.
	 * @return the server socket.
	 */
	public ServerSocket serverSocket() {
		return sct;
	}	
		
	/**
	 * Get the data socket.
	 * @return the data socket.
	 */
	public Socket dataSocket() {
		return socket;
	}
	
	/**
	 * Get the port for the server.
	 * @return the port of the server.
	 */
	public int getPort() {
		return sct.getLocalPort();
	}
		
	/**
	 * Send the time.
	 */
	public void time() {
		transmit("TIME "+System.currentTimeMillis());
	}

}
